from decimal import Decimal
from django import forms
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import smart_str
from product.models import Option, Product, ProductPriceLookup, OptionGroup, Price ,make_option_unique_id
from product.prices import get_product_quantity_price, get_product_quantity_adjustments
from satchmo_utils import cross_list
from satchmo_utils.unique_id import slugify
import datetime
import logging

SATCHMO_PRODUCT=True

def get_product_types():
    return ('ConfigurableProduct','ProductVariation')

def sorted_tuple(lst):
    ret = []
    for x in lst:
        if not x in ret:
            ret.append(x)
    ret.sort()
    return tuple(ret)

log = logging.getLogger('product.modules.configurable')

def get_all_options(obj, ids_only=False):
    """
    Returns all possible combinations of options for this products OptionGroups as a List of Lists.
    Ex:
    For OptionGroups Color and Size with Options (Blue, Green) and (Large, Small) you'll get
    [['Blue', 'Small'], ['Blue', 'Large'], ['Green', 'Small'], ['Green', 'Large']]
    Note: the actual values will be instances of Option instead of strings
    """
    sublist = []
    masterlist = []
    #Create a list of all the options & create all combos of the options
    for opt in obj.option_group.select_related().all():
        for value in opt.option_set.all():
            if ids_only:
                sublist.append(value.unique_id)
            else:
                sublist.append(value)
        masterlist.append(sublist)
        sublist = []
    results = cross_list(masterlist)
    return results

class ConfigurableProduct(models.Model):
    """
    Product with selectable options.
    This is a sort of virtual product that is visible to the customer, but isn't actually stocked on a shelf,
    the specific "shelf" product is determined by the selected options.
    """
    product = models.OneToOneField(Product, verbose_name=_("Product"), primary_key=True)
    option_group = models.ManyToManyField(OptionGroup, blank=True, verbose_name=_("Option Group"))
    create_subs = models.BooleanField(_("Create Variations"), default=False, help_text =_("Create ProductVariations for all this product's options.  To use this, you must first add an option, save, then return to this page and select this option."))

    def __init__(self, *args, **kwargs):
        super(ConfigurableProduct, self).__init__(*args, **kwargs)

    def _get_subtype(self):
        return 'ConfigurableProduct'

    def get_all_options(self):
        """
        Returns all possible combinations of options for this products OptionGroups as a List of Lists.
        Ex:
        For OptionGroups Color and Size with Options (Blue, Green) and (Large, Small) you'll get
        [['Blue', 'Small'], ['Blue', 'Large'], ['Green', 'Small'], ['Green', 'Large']]
        Note: the actual values will be instances of Option instead of strings
        """
        sublist = []
        masterlist = []
        #Create a list of all the options & create all combos of the options
        for opt in self.option_group.all():
            for value in opt.option_set.all():
                sublist.append(value)
            masterlist.append(sublist)
            sublist = []
        return cross_list(masterlist)

    def get_valid_options(self):
        """
        Returns unique_ids from get_all_options(), but filters out Options that this
        ConfigurableProduct doesn't have a ProductVariation for.
        """
        variations = self.productvariation_set.filter(product__active='1')
        active_options = [v.unique_option_ids for v in variations]
        all_options = get_all_options(self, ids_only=True)
        return [opt for opt in all_options if self._unique_ids_from_options(opt) in active_options]

    def create_all_variations(self):
        """
        Get a list of all the optiongroups applied to this object
        Create all combinations of the options and create variations
        """
        # Create a new ProductVariation for each combination.
        for options in self.get_all_options():
            self.create_variation(options)

    def create_variation(self, options, name=u"", sku=u"", slug=u""):
        """Create a productvariation with the specified options.
        Will not create a duplicate."""
        log.debug("Create variation: %s", options)
        variations = self.get_variations_for_options(options)

        if not variations:
            # There isn't an existing ProductVariation.
            if self.product:
                site = self.product.site
            else:
                site = self.site

            variant = Product(site=site, items_in_stock=0, name=name)
            optnames = [opt.value for opt in options]
            if not slug:
                slug = slugify(u'%s_%s' % (self.product.slug, u'_'.join(optnames)))

            while Product.objects.filter(slug=slug).count():
                slug = u'_'.join((slug, unicode(self.product.id)))

            variant.slug = slug

            log.info("Creating variation for [%s] %s", self.product.slug, variant.slug)
            variant.save()

            pv = ProductVariation(product=variant, parent=self)
            pv.save()

            for option in options:
                pv.options.add(option)

            pv.name = name
            pv.sku = sku
            pv.save()

        else:
            variant = variations[0].product
            log.debug("Existing variant: %s", variant)
            dirty = False
            if name and name != variant.name:
                log.debug("Updating name: %s --> %s", self, name)
                variant.name = name
                dirty = True
            if sku and sku != variant.sku:
                variant.sku = sku
                log.debug("Updating sku: %s --> %s", self, sku)
                dirty = True
            if slug:
                # just in case
                slug = slugify(slug)
            if slug and slug != variant.slug:
                variant.slug = slug
                log.debug("Updating slug: %s --> %s", self, slug)
                dirty = True
            if dirty:
                log.debug("Changed existing variant, saving: %s", variant)
                variant.save()
            else:
                log.debug("No change to variant, skipping save: %s", variant)

        return variant

    def _unique_ids_from_options(self, options):
        """
        Takes an iterable of Options (or str(Option)) and outputs a sorted tuple of
        option unique ids suitable for comparing to a productvariation.option_values
        """
        optionlist = []
        for opt in options:
            if isinstance(options[0], Option):
                opt = opt.unique_id
            optionlist.append(opt)

        return sorted_tuple(optionlist)

    def get_product_from_options(self, options):
        """
        Accepts an iterable of either Option object or a sorted tuple of
        options ids.
        Returns the product that matches or None
        """
        options = self._unique_ids_from_options(options)
        pv = None
        if hasattr(self, '_variation_cache'):
            pv =  self._variation_cache.get(options, None)
        else:
            for member in self.productvariation_set.all():
                if member.unique_option_ids == options:
                    pv = member
                    break
        if pv:
            return pv.product
        return None

    def get_variations_for_options(self, options):
        """
        Returns a list of existing ProductVariations with the specified options.
        """
        variations = ProductVariation.objects.filter(parent=self)
        for option in options:
            variations = variations.filter(options=option)
        return variations

    def add_template_context(self, context, request, selected_options, default_view_tax=False, **kwargs):
        """
        Add context for the product template.
        Return the updated context.
        """
        from product.utils import productvariation_details, serialize_options

        selected_options = self._unique_ids_from_options(selected_options)

        options = serialize_options(self, selected_options)
        if not 'options' in context:
            context['options'] = options
        else:
            curr = list(context['options'])
            curr.extend(list(options))
            context['options'] = curr

        details = productvariation_details(self.product, default_view_tax,
                                           request.user)
        if 'details' in context:
            context['details'].update(details)
        else:
            context['details'] = details

        return context

    def save(self, **kwargs):
        """
        Right now this only works if you save the suboptions, then go back and choose to create the variations.
        """
        super(ConfigurableProduct, self).save(**kwargs)
        if hasattr(self.product,'_sub_types'):
            del self.product._sub_types
        # Doesn't work with admin - the manipulator doesn't add the option_group
        # until after save() is called.
        if self.create_subs and self.option_group.count():
            self.create_all_variations()
            self.create_subs = False
            super(ConfigurableProduct, self).save(**kwargs)

        ProductPriceLookup.objects.smart_create_for_product(self.product)

    def get_absolute_url(self):
        return self.product.get_absolute_url()

    def setup_variation_cache(self):
        self._variation_cache = {}
        for member in self.productvariation_set.all():
            key = member.unique_option_ids
            self._variation_cache[key] = member

    class Meta:
        verbose_name = _("Configurable Product")
        verbose_name_plural = _("Configurable Products")

    def __unicode__(self):
        return self.product.slug


class ProductVariationManager(models.Manager):

    def by_parent(self, parent):
        """Get the list of productvariations which have the `product` as the parent"""
        return ProductVariation.objects.filter(parent=parent)

class ProductVariation(models.Model):
    """
    This is the real Product that is ordered when a customer orders a
    ConfigurableProduct with the matching Options selected

    """
    product = models.OneToOneField(Product, verbose_name=_('Product'), primary_key=True)
    options = models.ManyToManyField(Option, verbose_name=_('Options'))
    parent = models.ForeignKey(ConfigurableProduct, verbose_name=_('Parent'))

    objects = ProductVariationManager()

    def _get_fullPrice(self):
        """ Get price based on parent ConfigurableProduct """
        # allow explicit setting of prices.
        #qty_discounts = self.price_set.exclude(expires__isnull=False, expires__lt=datetime.date.today()).filter(quantity__lte=1)
        try:
            qty_discounts = Price.objects.filter(product__id=self.product.id).exclude(expires__isnull=False, expires__lt=datetime.date.today())
            if qty_discounts.count() > 0:
                # Get the price with the quantity closest to the one specified without going over
                return qty_discounts.order_by('-quantity')[0].dynamic_price

            if self.parent.product.unit_price is None:
                log.warn("%s: Unexpectedly no parent.product.unit_price", self)
                return None

        except AttributeError:
            pass

        # calculate from options
        return self.parent.product.unit_price + self.price_delta()

    unit_price = property(_get_fullPrice)

    def _get_optionName(self):
        "Returns the options in a human readable form"
        if self.options.count() == 0:
            return self.parent.verbose_name
        output = self.parent.verbose_name + " ( "
        numProcessed = 0
        # We want the options to be sorted in a consistent manner
        optionDict = dict([(sub.option_group.sort_order, sub) for sub in self.options.all()])
        for optionNum in optionDict.keys().sort():
            numProcessed += 1
            if numProcessed == self.options.count():
                output += optionDict[optionNum].name
            else:
                output += optionDict[optionNum].name + "/"
        output += " )"
        return output
    full_name = property(_get_optionName)

    def _optionkey(self):
        #todo: verify ordering
        optkeys = [smart_str(x) for x in self.options.values_list('value', flat=True).order_by('option_group__id')]
        return "::".join(optkeys)
    optionkey = property(fget=_optionkey)

    def _get_option_ids(self):
        """
        Return a sorted tuple of all the valid options for this variant.
        """
        qry = self.options.values_list('option_group__id', 'value').order_by('option_group')
        ret = [make_option_unique_id(*v) for v in qry]
        return sorted_tuple(ret)

    unique_option_ids = property(_get_option_ids)

    def _get_subtype(self):
        return 'ProductVariation'

    def _has_variants(self):
        return True
    has_variants = property(_has_variants)

    def _get_category(self):
        """
        Return the primary category associated with this product
        """
        return self.parent.product.category.all()[0]
    get_category = property(_get_category)

    def _check_optionParents(self):
        groupList = []
        for option in self.options.all():
            if option.option_group.id in groupList:
                return(True)
            else:
                groupList.append(option.option_group.id)
        return(False)

    def get_qty_price(self, qty, include_discount=True):
        if include_discount:
            price = get_product_quantity_price(
                self.product, qty,
                delta=self.price_delta(False),
                parent=self.parent.product)
        else:
            adjustment = get_product_quantity_adjustments(self, qty, parent=self.parent.product)
            if adjustment.price is not None:
                price = adjustment.price.price + self.price_delta(True)
            else:
                price = None

        return price

    def get_qty_price_list(self):
        """Return a list of tuples (qty, price)"""
        prices = Price.objects.filter(product__id=self.product.id).exclude(expires__isnull=False, expires__lt=datetime.date.today())
        if prices.count() > 0:
            # prices directly set, return them
            pricelist = [(price.quantity, price.dynamic_price) for price in prices]
        else:
            prices = self.parent.product.get_qty_price_list()
            price_delta = self.price_delta()

            pricelist = [(qty, price+price_delta) for qty, price in prices]

        return pricelist

    def _is_shippable(self):
        product = self.product
        parent = self.parent.product
        return (product.shipclass == 'YES' or
                (product.shipclass == "DEFAULT"
                 and parent.shipclass in ("DEFAULT", "YES"))
                )

    is_shippable = property(fget=_is_shippable)

    def isValidOption(self, field_data, all_data):
        raise forms.ValidationError(_("Two options from the same option group cannot be applied to an item."))

    def price_delta(self, include_discount=True):
        # TODO: deltas aren't taken into account by satchmo_price_query
        price_delta = Decimal("0.00")
        for option in self.options.all():
            if option.price_change:
                price_delta += Decimal(option.price_change)
        return price_delta

    def save(self, **kwargs):
        # don't save if the product is a configurableproduct
        if "ConfigurableProduct" in self.product.get_subtypes():
            log.warn("cannot add a productvariation subtype to a product which already is a configurableproduct. Aborting")
            return

        pvs = ProductVariation.objects.filter(parent=self.parent)
        pvs = pvs.exclude(product=self.product)
        for pv in pvs:
            if pv.unique_option_ids == self.unique_option_ids:
                return # Don't allow duplicates

        if not self.product.name:
            # will force calculation of default name
            self.name = ""

        super(ProductVariation, self).save(**kwargs)
        ProductPriceLookup.objects.smart_create_for_product(self.product)

    def _set_name(self, name):
        if not name:
            name = self.parent.product.name
            options = [option.name for option in self.options.order_by("option_group")]
            if options:
                name = u'%s (%s)' % (name, u'/'.join(options))
            log.debug("Setting default name for ProductVariant: %s", name)

        self.product.name = name
        self.product.save()

    def _get_name(self):
        return self.product.name

    name = property(fset=_set_name, fget=_get_name)

    def _set_sku(self, sku):
        if not sku:
            sku = self.product.slug
        self.product.sku = sku
        self.product.save()

    def _get_sku(self):
        return self.product.sku

    sku = property(fset=_set_sku, fget=_get_sku)

    def get_absolute_url(self):
        return self.product.get_absolute_url()

    class Meta:
        verbose_name = _("Product variation")
        verbose_name_plural = _("Product variations")

    def __unicode__(self):
        return self.product.slug



"""Widgets used to edit menubars

They are very similar to the real GTK+ ones but a) allow in-place edition
of the menubar and b) has less advanced features like multiscreen/monitor
support or keyboard navigation
"""
# Copyright (C) 2006 by Nokia Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
import xml.dom

import gobject
import gtk
import pango

from kiwi.ui.dialogs import error

from gazpacho.actioneditor import CommandAddRemoveAction, CommandEditAction
from gazpacho.commandmanager import command_manager
from gazpacho.gaction import GAction
from gazpacho.i18n import _
from gazpacho.uieditor import CommandUpdateUIDefinitions
from gazpacho.widgets.base.menueditor import contextmenu
from gazpacho.widgets.base.menueditor import gtkutils

MENU_SHELL_TIMEOUT = 500

# Backward compatibility with GTK+ 2.6.X and below
if gtk.gtk_version < (2, 8, 0):
    (PACK_DIRECTION_LTR,
     PACK_DIRECTION_RTL,
     PACK_DIRECTION_TTB,
     PACK_DIRECTION_BTT) = list(range(4))
else:
    PACK_DIRECTION_LTR = gtk.PACK_DIRECTION_LTR
    PACK_DIRECTION_RTL = gtk.PACK_DIRECTION_RTL
    PACK_DIRECTION_TTB = gtk.PACK_DIRECTION_TTB
    PACK_DIRECTION_BTT = gtk.PACK_DIRECTION_BTT


class MenuShell(gtk.Container):
    """Base class for MenuBar and Menu.

    It does children management and some low level event handlers.
    """

    # This should be a callable that returns a gtk.Menu ready to pop up
    context_menu_factory = None

    # default placement of submenus attached to the menu item children
    submenu_placement = gtk.TOP_BOTTOM

    def __init__(self, gadget):
        """Constructor

        gadget must be a Gazpacho Gadget representing the menubar it's
        being edited.
        """
        gtk.Container.__init__(self)

        self.gadget = gadget # Gazpacho associated gadget

        self.children = []
        self.active_menu_item = None
        self.parent_menu_shell = None

        self.active = False
        self.have_grab = False
        self.have_xgrab = False
        self.ignore_leave = False
        self.ignore_enter = False

        self.button = 0
        self.activate_time = 0

        self.take_focus = True

    def append(self, child):
        """Add child at the end of the children list"""
        self.insert(child, len(self.children))

    def prepend(self, child):
        """Add child at the begining of the children list"""
        self.insert(child, 0)

    def insert(self, child, position):
        """Add the children to the list at the specific position"""
        self.children.insert(position, child)
        child.set_parent(self)

    def to_xml(self, document):
        """Returns a XML node with the representation of this menushell"""
        tag = self.__class__.__name__.lower()
        element = document.createElement(tag)
        for child_node in self._get_children_nodes(document):
            element.appendChild(child_node)
        return element

    def _get_children_nodes(self, document):
        """Helper method to the to_xml method

        Returns a list of xml nodes corresponding to our children
        """
        ret = []
        for child in self.children:
            if child.submenu:
                node = child.submenu.to_xml(document)
            else:
                node = child.to_xml(document)

            if node:
                ret.append(node)
        return ret

    def load(self, xml_node):
        """Inverse to_xml method: creates a menushell from a xml node"""
        uim = self.gadget.project.uim
        default_action_group = uim.get_action_group('DefaultActions')
        for node in xml_node.childNodes:
            if node.nodeType != node.ELEMENT_NODE:
                continue

            name = node.getAttribute('action')
            action = default_action_group.get_action(name)
            if action:
                menuitem = MenuItem(action.label,
                                    self.gadget, default_action_group)
                menuitem.set_action(action)
            else:
                menuitem = SeparatorMenuItem()

            parent_is_menubar = isinstance(self, MenuBar)

            if node.hasChildNodes() or parent_is_menubar:
                submenu = Menu(self.gadget)
                submenu.load(node)
                menuitem.set_submenu(submenu)

            menuitem.show_all()

            self.insert(menuitem, len(self.children) - 1)

    # GtkWidget methods
    def do_realize(self):
        """Overrides widget_class->realize"""
        self.set_flags(self.flags() | gtk.REALIZED)

        mask = (gtk.gdk.EXPOSURE_MASK | gtk.gdk.BUTTON_PRESS_MASK |
                gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.KEY_PRESS_MASK |
                gtk.gdk.ENTER_NOTIFY_MASK | gtk.gdk.LEAVE_NOTIFY_MASK)

        self.window = gtk.gdk.Window(
            self.get_parent_window(),
            x=self.allocation.x,
            y=self.allocation.y,
            width=self.allocation.width,
            height=self.allocation.height,
            window_type=gtk.gdk.WINDOW_CHILD,
            wclass=gtk.gdk.INPUT_OUTPUT,
            event_mask=self.get_events() | mask)

        self.window.set_user_data(self)
        self.style.attach(self.window)

        self.style.set_background(self.window, gtk.STATE_NORMAL)

    def do_button_press_event(self, event):
        """Overrides widget_class->button_press_event

        The reason for overriding this method is to popup the context menu
        with options to edit the selected item.
        """
        if event.button == 3:
            if self.context_menu_factory:
                menu_item = gtkutils.get_event_widget(event)
                if (menu_item and isinstance(menu_item, MenuItem) and
                    not isinstance(menu_item, (AddMenuItem, AddActionItem))):
                    popup = self.context_menu_factory(menu_item)
                    popup.popup(None, None, None, event.button, event.time)
                    return True

        elif self.parent_menu_shell:
            return self.parent_menu_shell.event(event)

        elif not self.active or self.button != 0:
            self._activate()
            self.button = event.button
            menu_item = self.get_item(event)
            if menu_item and menu_item.is_selectable():
                if (menu_item.parent == self and
                    menu_item != self.active_menu_item):
                    self.activate_time = event.time
                    self.select_item(menu_item)
        else:
            widget = gtkutils.get_event_widget(event)
            if widget == self:
                self.deactivate()

        return True

    def do_button_release_event(self, event):
        """Overrides widget_class->button_release_event"""
        if self.active:
            if self.button != 0 and event.button != self.button:
                self.button = 0
                if self.parent_menu_shell:
                    return self.parent_menu_shell.event(event)
            self.button = 0
            menu_item = self.get_item(event)
            deactivate = True

            if event.time - self.activate_time > MENU_SHELL_TIMEOUT:
                if (menu_item and self.active_menu_item == menu_item and
                    menu_item.is_selectable()):
                    if menu_item.submenu is None:
                        self.activate_menu_item(menu_item)
                        return True
                    else:
                        menu_item.select()
                        return True
                elif menu_item and not menu_item.is_selectable():
                    deactivate = False
                elif self.parent_menu_shell:
                    self.active = True
                    self.parent_menu_shell.event(event)
                    return True

                if menu_item and self.active_menu_item == menu_item:
                    deactivate = False
            else: # a very fast press-release
                self.activate_time = 0
                deactivate = False

            if deactivate:
                self.deactivate()

        return True

    def do_enter_notify_event(self, event):
        """Overrides widget_class->enter_notify_event"""
        if self.active:
            menu_item = gtkutils.get_event_widget(event)
            if (not menu_item or (isinstance(menu_item, MenuItem) and
                                  not menu_item.is_selectable())):
                return True

            if (menu_item.parent == self and
                self.active_menu_item != menu_item and
                isinstance(menu_item, MenuItem)):
                if self.ignore_enter:
                    return True

                if (event.detail != gtk.gdk.NOTIFY_INFERIOR and
                    menu_item.state != gtk.STATE_PRELIGHT and
                    not isinstance(menu_item, AddMenuItem)):
                    self.select_item(menu_item)

            elif self.parent_menu_shell:
                self.parent_menu_shell.event(event)
        return True

    def do_leave_notify_event(self, event):
        """Overrides widget_class->leave_notify_event"""
        if gtkutils.widget_visible(self):
            event_widget = gtkutils.get_event_widget(event)

            if (not event_widget or
                not isinstance(event_widget, MenuItem)):
                return True

            menu_item = event_widget

            if not menu_item.is_selectable():
                return True

            if (self.active_menu_item == menu_item and
                menu_item.submenu is None):
                if (event.detail != gtk.gdk.NOTIFY_INFERIOR and
                    menu_item.state != gtk.STATE_NORMAL):
                    self.deselect()

            elif self.parent_menu_shell:
                self.parent_menu_shell.event(event)

        return True

    # GtkContainer methods
    def do_add(self, widget):
        """Overrides container_class->add"""
        self.append(widget)

    def do_remove(self, widget):
        """Overrides container_class->remove"""
        was_visible = gtkutils.widget_visible(widget)
        self.children.remove(widget)

        if widget == self.active_menu_item:
            self.active_menu_item.deselect()
            self.active_menu_item = None

        widget.unparent()

        if was_visible:
            self.queue_resize()

    def do_forall(self, internal, callback, data):
        """Overrides container_class->forall"""
        for child in self.children:
            callback(child, data)

    def _activate(self):
        """Do a GTK+ grab and change the internal state to active"""
        if not self.active:
            self.grab_add()
            self.have_grab = True
            self.active = True

    def deactivate(self):
        """Change the state to not active and deselect the active item
        if there is one. Also ungrab at the GTK+ and GDK level.
        """
        if self.active:
            self.button = 0
            self.active = False
            self.activate_time = 0

            if self.active_menu_item:
                self.active_menu_item.deselect()
                self.active_menu_item = None

            if self.have_grab:
                self.have_grab = False
                self.grab_remove()

            if self.have_xgrab:
                display = self.get_display()
                self.have_xgrab = False
                display.pointer_ungrab()
                display.keyboard_ungrab()

    def is_item(self, child):
        """Helper method to get_item. True if child is a direct or indirect
        child of self. It can be a child of a submenu (indirect).
        """
        parent = child.parent
        while parent and isinstance(parent, MenuShell):
            if parent == self:
                return True
            parent = parent.parent_menu_shell

        return False

    def get_item(self, event):
        """Return the menu item over which the event occured"""
        menu_item = gtkutils.get_event_widget(event)

        while (menu_item and not isinstance(menu_item, MenuItem)):
            menu_item = menu_item.parent

        if (menu_item and self.is_item(menu_item)):
            return menu_item

    def can_select_item(self, menu_item):
        """True if menu_item can be selected"""
        if (not self.active) or (not self.active_menu_item == menu_item):
            return True
        return False

    def select_item(self, menu_item):
        """Selects menu_item if that is possible.

        This will probably mean that a submenu is popup.
        """
        if self.can_select_item(menu_item):
            self.deselect()

            if not menu_item.is_selectable():
                return

            self.active_menu_item = menu_item

            pack_dir = self.get_pack_direction()
            if (pack_dir == PACK_DIRECTION_TTB or
                pack_dir == PACK_DIRECTION_BTT):
                self.active_menu_item.set_placement(gtk.LEFT_RIGHT)
            else:
                placement = self.__class__.submenu_placement
                self.active_menu_item.set_placement(placement)

            self.active_menu_item.select()

    def deselect(self):
        """Unselect the active menu item if there is one.

        This will probably means that a submenu is popdown.
        """
        if self.active_menu_item:
            self.active_menu_item.deselect()
            self.active_menu_item = None

    def get_pack_direction(self):
        """Returns the pack direction"""
        return PACK_DIRECTION_LTR

    def activate_menu_item(self, menu_item):
        """Activate the menu_item. This will mean start editing its label"""
        # first we end editing all the children
        for child in self.children:
            if child.editing:
                child.end_editing()
        menu_item.activate()

gobject.type_register(MenuShell)

#
# MenuBar widget
#

BORDER_SPACING = 0
DEFAULT_IPADDING = 1

class MenuBar(MenuShell):
    """Editable MenuBar

    You can add menus to this bar by clicking on the special
    'Click to create menu' item.

    You can edit existing menus by right clicking on them and using the
    context menu.

    You can edit submenus by poping up them like a standard Menu Bar
    """

    context_menu_factory = contextmenu.MenuBarContextMenu
    submenu_placement = gtk.TOP_BOTTOM

    def __init__(self, gadget):
        """Constructor. Initialize the bar and put an AddMenuItem item"""
        MenuShell.__init__(self, gadget)

        self.child_pack_direction = PACK_DIRECTION_LTR
        self.pack_direction = PACK_DIRECTION_LTR

        item = AddMenuItem(self.gadget)
        item.show()
        self.append(item)

    def to_xml(self, document):
        """See MenuShell.to_xml"""
        element = super(MenuBar, self).to_xml(document)
        element.setAttribute('name', self.gadget.name)
        element.setAttribute('action', self.gadget.name)
        return element

    def update_ui(self):
        """Updates the UI definition for the gadget this MenuBar is editing

        It uses a Gazpacho command to update the UI definition in the project
        UI Manager.
        """
        dom = xml.dom.getDOMImplementation()
        document = dom.createDocument(None, None, None)
        ui_string = self.to_xml(document).toxml()
        cmd = CommandUpdateUIDefinitions(self.gadget, ui_string,
                                         'initial-state', False)
        command_manager.execute(cmd, self.gadget.project)

    def do_size_request(self, requisition):
        """Overrides widget_class->size_request"""
        requisition.width = 0
        requisition.height = 0

        if gtkutils.widget_visible(self):

            nchildren = 0

            for child in self.children:
                if gtkutils.widget_visible(child):
                    child.show_submenu_indicator = False
                    c_width, c_height = child.size_request()
                    toggle_size = child.toggle_size_request()

                    if (self.child_pack_direction == PACK_DIRECTION_LTR or
                        self.child_pack_direction == PACK_DIRECTION_RTL):
                        c_width += toggle_size
                    else:
                        c_height += toggle_size

                    if (self.pack_direction == PACK_DIRECTION_LTR or
                        self.pack_direction == PACK_DIRECTION_RTL):
                        requisition.width += c_width
                        requisition.height = max(requisition.height, c_height)
                    else:
                        requisition.width = max(requisition.width, c_width)
                        requisition.height += c_height

                    nchildren += 1

            ipadding = self.style_get_property('internal-padding')

            margin = self.border_width + ipadding + BORDER_SPACING * 2
            requisition.width += margin
            requisition.height += margin

            if self.get_shadow_type() != gtk.SHADOW_NONE:
                requisition.width += self.style.xthickness * 2
                requisition.height += self.style.ythickness * 2


    def do_size_allocate(self, allocation):
        """Overrides widget_class->size_allocate"""
        child_allocation = gtk.gdk.Rectangle()
        direction = self.get_direction()
        self.allocation = allocation

        if gtkutils.widget_realized(self):
            self.window.move_resize(allocation.x, allocation.y,
                                    allocation.width, allocation.height)

        ipadding = self.style_get_property('internal-padding')

        if self.children:
            child_allocation.x = self.border_width + ipadding + BORDER_SPACING
            child_allocation.y = self.border_width + ipadding + BORDER_SPACING

            if self.get_shadow_type() != gtk.SHADOW_NONE:
                child_allocation.x += self.style.xthickness
                child_allocation.y += self.style.ythickness

            if (self.pack_direction == PACK_DIRECTION_LTR or
                self.pack_direction == PACK_DIRECTION_RTL):
                child_allocation.height = max(1, allocation.height -
                                              child_allocation.y * 2)
                ltr_x = child_allocation.x

                for child in self.children:
                    toggle_size = child.toggle_size_request()
                    cr_width, cr_height = child.get_child_requisition()

                    if (self.child_pack_direction == PACK_DIRECTION_LTR or
                        self.child_pack_direction == PACK_DIRECTION_RTL):
                        cr_width += toggle_size
                    else:
                        cr_height += toggle_size

                    if gtkutils.widget_visible(child):
                        if ((direction == gtk.TEXT_DIR_LTR) ==
                            (self.pack_direction == PACK_DIRECTION_LTR)):
                            child_allocation.x = ltr_x
                        else:
                            child_allocation.x = (allocation.width
                                                  - cr_width - ltr_x)
                        child_allocation.width = cr_width
                        child.toggle_size_allocate(toggle_size)
                        child.size_allocate(child_allocation)

                        ltr_x += child_allocation.width
            else:
                child_allocation.width = max(1, allocation.width -
                                             child_allocation.x * 2)
                ltr_y = child_allocation.y

                for child in self.children:
                    toggle_size = child.toggle_size_request()
                    cr_width, cr_height = child.get_child_requisition()

                    if (self.child_pack_direction == PACK_DIRECTION_LTR or
                        self.child_pack_direction == PACK_DIRECTION_RTL):
                        cr_width += toggle_size
                    else:
                        cr_height += toggle_size

                    if gtkutils.widget_visible(child):
                        if ((direction == gtk.TEXT_DIR_LTR) ==
                            (self.pack_direction == PACK_DIRECTION_TTB)):
                            child_allocation.y = ltr_y
                        else:
                            child_allocation.y = (allocation.height
                                                  - cr_height - ltr_y)
                        child_allocation.height = cr_height
                        child.toggle_size_allocate(toggle_size)
                        child.size_allocate(child_allocation)

                        ltr_y += child_allocation.height

    def paint(self, area):
        """Helper method for the expose event handler"""
        if gtkutils.widget_drawable(self):
            border = self.border_width
            self.style.paint_box(self.window, self.state,
                                 self.get_shadow_type(), area, self, 'menubar',
                                 border, border,
                                 self.allocation.width - border * 2,
                                 self.allocation.height - border * 2)

    def do_expose_event(self, event):
        """Overrides widget_class->expose_event"""
        if gtkutils.widget_drawable(self):
            self.paint(event.area)

            MenuShell.do_expose_event(self, event)

        return False

    def get_shadow_type(self):
        """Returns the shadow used to draw the widget"""
        shadow_type = self.style_get_property('shadow-type')
        if not shadow_type:
            shadow_type = gtk.SHADOW_OUT
        return shadow_type

    def get_pack_direction(self):
        """Returns the pack direction"""
        return self.pack_direction

    def set_pack_direction(self, direction):
        """Set the pack direction and update the widget if necesary"""
        if self.pack_direction != direction:
            self.pack_direction = direction
            self.queue_resize()
            for child in self.children:
                child.queue_resize()

    def get_child_pack_direction(self):
        """Retrusn the pack direction for children"""
        return self.child_pack_direction

    def set_child_pack_direction(self, direction):
        """Set the children pack direction and update the widget if
        necessary"""
        if self.child_pack_direction != direction:
            self.child_pack_direction = direction
            self.queue_resize()
            for child in self.children:
                child.queue_resize()

gobject.type_register(MenuBar)
gtk.widget_class_install_style_property(MenuBar,
                                        ('shadow-type',
                                         gtk.ShadowType,
                                         'Shadow Type',
                                         'Style of bevel around the menubar',
                                         gtk.SHADOW_OUT,
                                         gobject.PARAM_READABLE))
desc = 'Amount of border space between the menubar shadow and the menu items'
gtk.widget_class_install_style_property(MenuBar,
                                        ('internal-padding',
                                         gobject.TYPE_INT,
                                         'Internal Padding',
                                         desc,
                                         0, 100, DEFAULT_IPADDING,
                                         gobject.PARAM_READABLE))

#
# Menu widget
#

DEFAULT_POPUP_DELAY = 225
DEFAULT_POPDOWN_DELAY = 1000

ATTACH_INFO_KEY = "gtk-menu-child-attach-info-key"
ATTACHED_MENUS = "gtk-attached-menus"

ATTACH_DATA_KEY = "gtk-menu-attach-data"

class Menu(MenuShell):
    """Editable Menu

    You can add actions to this menu by clicking on the special
    'Click to create action' item.

    You can edit existing menus by right clicking on them and using the
    context menu.

    You can edit submenus by poping up them like a standard Menu
    """

    context_menu_factory = contextmenu.MenuContextMenu
    submenu_placement = gtk.LEFT_RIGHT

    def __init__(self, gadget):
        """Constructor. Initialize the menu and put an AddActionItem item"""
        MenuShell.__init__(self, gadget)

        self.parent_menu_item = None
        self.old_active_menu_item = None
        self.position_func = None
        self.position_func_data = None
        self.toggle_size = 0

        self.toplevel = gobject.new(gtk.Window,
                                    type=gtk.WINDOW_POPUP,
                                    child=self)
        self.toplevel.connect('event', menu_window_event, self)
        self.toplevel.connect('size_request', menu_window_size_request, self)
        self.toplevel.set_resizable(False)
        self.toplevel.set_mnemonic_modifier(0)

        self.view_window = None
        self.bin_window = None

        self.seen_item_enter = False

        self.have_position = False
        self.x = 0
        self.y = 0

        # info used for the table
        self.heights = []

        self.monitor_num = 0

        self.have_layout = False
        self.n_rows = 0
        self.n_columns = 0

        item = AddActionItem(self.gadget)
        item.show_all()
        self.append(item)

    def to_xml(self, document):
        """See MenuShell.to_xml"""
        menuitem = self.get_attach_widget()
        if menuitem and menuitem.action:
            element = super(Menu, self).to_xml(document)
            element.setAttribute('name', menuitem.action.name)
            element.setAttribute('action', menuitem.action.name)
            return element

    def queue_resize(self):
        """Invalidates the current layout before calling the
        real queue_resize"""
        self.have_layout = False
        MenuShell.queue_resize(self)

    def ensure_layout(self):
        """Computes the table attachment for every child"""
        if not self.have_layout:

            # find extents of gridded portion
            max_right_attach = 1
            max_bottom_attach = 0

            for child in self.children:
                a_info = get_attach_info(child)

                if a_info.is_grid_attached():
                    max_bottom_attach = max(max_bottom_attach,
                                            a_info.bottom_attach)
                    max_right_attach = max(max_right_attach,
                                           a_info.right_attach)

            # find empty rows
            row_occupied = [False for i in range(max_bottom_attach)]
            for child in self.children:
                a_info = get_attach_info(child)

                if a_info.is_grid_attached():
                    for i in range(a_info.top_attach, a_info.bottom_attach):
                        row_occupied[i] = True

            # lay non-grid items out in those rows
            current_row = 0
            for child in self.children:
                a_info = get_attach_info(child)
                if not a_info.is_grid_attached():
                    while (current_row < max_bottom_attach and
                           row_occupied[current_row]):
                        current_row += 1

                    a_info.effective_left_attach = 0
                    a_info.effective_right_attach = max_right_attach
                    a_info.effective_top_attach = current_row
                    a_info.effective_bottom_attach = current_row + 1
                    current_row += 1

                else:
                    a_info.effective_left_attach = a_info.left_attach
                    a_info.effective_right_attach = a_info.right_attach
                    a_info.effective_top_attach = a_info.top_attach
                    a_info.effective_bottom_attach = a_info.bottom_attach
            self.n_rows = max(current_row, max_bottom_attach)
            self.n_columns = max_right_attach
            self.have_layout = True

    def get_n_columns(self):
        """Returns the number of columns (usually 1, 2 or 3)"""
        self.ensure_layout()
        return self.n_columns

    def get_n_rows(self):
        """Returns the number of rows"""
        self.ensure_layout()
        return self.n_rows

    def get_effective_child_attach(self, child):
        """Return the real table attachments for the child"""
        self.ensure_layout()

        a_info = get_attach_info(child)
        return (a_info.effective_left_attach, a_info.effective_right_attach,
                a_info.effective_top_attach, a_info.effective_bottom_attach)

    def do_destroy(self):
        """Overrides widget_class->destroy doing some extra cleanups"""
        data = self.get_data(ATTACH_DATA_KEY)
        if data:
            self.detach()

        if self.old_active_menu_item:
            del self.old_active_menu_item
            self.old_active_menu_item = None

        if self.toplevel:
            self.toplevel.destroy()

        if self.heights:
            self.heights = []

        MenuShell.do_destroy(self)

    def attach_to_widget(self, attach_widget, detacher_func):
        """Atach the menu to another widget"""
        data = self.get_data(ATTACH_DATA_KEY)
        if data:
            print(('WARNING: attach_to_widget(): menu already attached to %s' %
                   data.attach_widget))
            return

        data = _MenuAttachData()
        data.attach_widget = attach_widget

        data.detacher = detacher_func
        self.set_data(ATTACH_DATA_KEY, data)
        menu_list = attach_widget.get_data(ATTACHED_MENUS)
        if not menu_list:
            menu_list = []
        if not self in menu_list:
            menu_list.append(self)

        attach_widget.set_data(ATTACHED_MENUS, menu_list)

        if self.state != gtk.STATE_NORMAL:
            self.set_state(gtk.STATE_NORMAL)

    def get_attach_widget(self):
        """Return the widget attached to the menu"""
        data = self.get_data(ATTACH_DATA_KEY)
        if data:
            return data.attach_widget

    def detach(self):
        """Isolates the menu again"""
        data = self.get_data(ATTACH_DATA_KEY)
        if not data:
            print('WARNING: menu is not attached')
            return

        self.set_data(ATTACH_DATA_KEY, None)
        if data.detacher:
            data.detacher(data.attach_widget, self)

        menu_list = data.attach_widget.get_data(ATTACHED_MENUS)
        menu_list.remove(self)
        if menu_list:
            data.attach_widget.set_data(ATTACHED_MENUS, menu_list)
        else:
            data.attach_widget.set_data(ATTACHED_MENUS, None)

        if gtkutils.widget_realized(self):
            self.unrealize()

    def do_remove(self, child):
        """Overrides container_class->remove"""
        # clear out old_active_menu_item if it matches the item we are removing
        if self.old_active_menu_item == child:
            self.old_active_menu_item = None

        MenuShell.do_remove(self, child)
        child.set_data(ATTACH_INFO_KEY, None)

        self.queue_resize()

    def insert(self, child, position):
        """Add a menuitem using the bin_window as the parent window"""
        if gtkutils.widget_realized(self):
            child.set_parent_window(self.bin_window)

        MenuShell.insert(self, child, position)

        self.queue_resize()

    def popup(self, parent_menu_shell, parent_menu_item, func, data, button,
              activate_time):
        """Shows the menu and mapped it on screen on the appropiate place"""

        self.parent_menu_shell = parent_menu_shell
        self.seen_item_enter = False

        # find the last viewable ancestor, and make an X grab on it
        parent = self
        xgrab_shell = None
        while parent:
            viewable = True

            tmp = parent

            while tmp:
                if not gtkutils.widget_mapped(tmp):
                    viewable = False
                    break
                tmp = tmp.parent

            if viewable:
                xgrab_shell = parent

            parent = parent.parent_menu_shell

        grab_keyboard = self.take_focus
        self.toplevel.set_accept_focus(grab_keyboard)

        if xgrab_shell and xgrab_shell != self:
            if gtkutils.popup_grab_on_window(xgrab_shell.window,
                                             activate_time, grab_keyboard):
                self.have_xgrab = True
        else:
            xgrab_shell = self
            transfer_window = self.grab_transfer_window_get()
            if gtkutils.popup_grab_on_window(transfer_window,
                                             activate_time, grab_keyboard):
                self.have_xgrab = True

        if not self.have_xgrab:
            self.parent_menu_shell = None
            self.grab_transfer_window_destroy()
            return

        self.active = True
        self.button = button

        current_event = gtk.get_current_event()
        if current_event:
            if ((current_event.type != gtk.gdk.BUTTON_PRESS) and
                (current_event.type != gtk.gdk.ENTER_NOTIFY)):
                self.ignore_enter = True
        else:
            self.ignore_enter = True

        parent_toplevel = None
        if parent_menu_shell:
            parent_toplevel = parent_menu_shell.get_toplevel()
        else:
            attach_widget = self.get_attach_widget()
            if attach_widget:
                parent_toplevel = attach_widget.get_toplevel()

        if parent_toplevel and isinstance(parent_toplevel, gtk.Window):
            self.toplevel.set_transient_for(parent_toplevel)

        self.parent_menu_item = parent_menu_item
        self.position_func = func
        self.position_func_data = data
        self.activate_time = activate_time

        self.show()

        self.position()

        alloc = gtk.gdk.Rectangle()
        alloc.width, alloc.height = self.toplevel.size_request()
        self.toplevel.size_allocate(alloc)

        self.realize()

        self.toplevel.show()

        if xgrab_shell == self:
            gtkutils.popup_grab_on_window(self.window, activate_time,
                                          grab_keyboard)

        self.grab_add()

    def popdown(self):
        """Hides the menu releasing any grab previously adquired"""
        self.parent_menu_shell = None
        self.active = False
        self.ignore_enter = False

        self.have_position = False

        if self.active_menu_item:
            self.old_active_menu_item = self.active_menu_item

        self.deselect()

        self.toplevel.hide()
        self.toplevel.set_transient_for(None)

        self.window.get_display().pointer_ungrab(0)
        gtk.gdk.keyboard_ungrab(0)

        self.hide()

        self.have_xgrab = False
        self.grab_remove()

        self.grab_transfer_window_destroy()

    def get_active(self):
        """Returns the active menu item"""
        if not self.old_active_menu_item:

            for child in self.children:
                if child.child:
                    self.old_active_menu_item = child

        return self.old_active_menu_item

    def set_active(self, index):
        """Sets the active menu item"""
        if index < len(self.children):
            child = self.children[index]
            if child.child:
                self.old_active_menu_item = child

    def do_can_activate_accel(self, signal_id):
        """Override widget_class->can_activate_accel"""
        awidget = self.get_attach_widget()
        if awidget:
            return awidget.can_activate_accel(signal_id)
        else:
            return gtkutils.widget_is_sensitive(self)

    def reposition(self):
        """Position the menu again"""
        if gtkutils.widget_drawable(self):
            self.position()

    def get_toplevel(self):
        """Return the toplevel widget this menu is related to going up through
        the menu hierarchy recursively"""
        attach = self.get_attach_widget()
        if isinstance(attach, MenuItem):
            attach = attach.parent

        if isinstance(attach, Menu):
            return attach.get_toplevel()
        elif isinstance(attach, gtk.Widget):
            toplevel = attach.get_toplevel()
            if gtkutils.widget_toplevel(toplevel):
                return toplevel

    def reorder_child(self, child, position):
        """Change the child position"""
        if child in self.children:
            self.children.remove(child)
            self.children.insert(child, position)

            self.queue_resize()

    def do_style_set(self, previous_style):
        """Sets the style changing the background of the 3 gdk windows"""
        if gtkutils.widget_realized(self):
            self.style.set_background(self.bin_window, gtk.STATE_NORMAL)
            self.style.set_background(self.view_window, gtk.STATE_NORMAL)
            self.style.set_background(self.window, gtk.STATE_NORMAL)

    def do_realize(self):
        """Overrides widget_class->realize"""
        self.set_flags(self.flags() | gtk.REALIZED)
        mask = (gtk.gdk.EXPOSURE_MASK | gtk.gdk.KEY_PRESS_MASK |
                gtk.gdk.ENTER_NOTIFY_MASK | gtk.gdk.LEAVE_NOTIFY_MASK)
        attrs = {
            'window_type': gtk.gdk.WINDOW_CHILD,
            'x': self.allocation.x,
            'y': self.allocation.y,
            'width': self.allocation.width,
            'height': self.allocation.height,
            'wclass': gtk.gdk.INPUT_OUTPUT,
            'event_mask': self.get_events() | mask,
            }
        self.window = gtk.gdk.Window(self.get_parent_window(), **attrs)
        self.window.set_user_data(self)

        border_width = self.border_width
        vertical_padding = self.style_get_property('vertical-padding')
        horizontal_padding = self.style_get_property('horizontal-padding')

        attrs['x'] = border_width + self.style.xthickness + horizontal_padding
        attrs['y'] = border_width + self.style.ythickness + vertical_padding
        attrs['width'] = max(1, self.allocation.width - attrs['x'] * 2)
        attrs['height'] = max(1, self.allocation.height - attrs['y'] * 2)

        self.view_window = gtk.gdk.Window(self.window, **attrs)
        self.view_window.set_user_data(self)

        attrs['x'] = 0
        attrs['y'] = 0
        attrs['width'] = max(1, self.allocation.width -
                             (border_width +
                              self.style.xthickness +
                              horizontal_padding) * 2)
        attrs['height'] = max(1, self.allocation.height -
                              (border_width +
                               self.style.ythickness +
                               vertical_padding) * 2)

        self.bin_window = gtk.gdk.Window(self.view_window, **attrs)
        self.bin_window.set_user_data(self)

        for child in self.children:
            child.set_parent_window(self.bin_window)

        self.set_style(self.style.attach(self.window))
        self.style.set_background(self.bin_window, gtk.STATE_NORMAL)
        self.style.set_background(self.view_window, gtk.STATE_NORMAL)
        self.style.set_background(self.window, gtk.STATE_NORMAL)

        self.bin_window.show()
        self.view_window.show()

    def grab_transfer_window_get(self):
        """Gets (creating if necessary) a window used to grab"""
        window = self.get_data('gtk-menu-transfer-window')
        if not window:
            window = gtk.gdk.Window(
                self.get_root_window(), x=-100, y=-100, width=10, height=10,
                window_type=gtk.gdk.WINDOW_TEMP, wclass=gtk.gdk.INPUT_ONLY,
                override_redirect=True, event_mask=0)
            window.set_user_data(self)
            window.show()
            self.set_data('gtk-menu-transfer-window', window)

        return window

    def grab_transfer_window_destroy(self):
        """Destroys the grab transfer window"""
        window = self.get_data('gtk-menu-transfer-window')
        if window:
            window.set_user_data(None)
            window.destroy()
            self.set_data('gtk-menu-transfer-window', None)

    def do_unrealize(self):
        """Overrides widget_class->unrealize"""
        self.grab_transfer_window_destroy()

        self.view_window.set_user_data(None)
        self.view_window.destroy()
        self.view_window = None

        self.bin_window.set_user_data(None)
        self.bin_window.destroy()
        self.bin_window = None

        MenuShell.do_unrealize(self)

    def do_size_request(self, requisition):
        """Overrides widget_class->size_request"""
        requisition.width = 0
        requisition.height = 0

        max_toggle_size = 0
        max_accel_width = 0

        n_rows = self.get_n_rows()
        self.heights = [0 for i in range(n_rows)]

        for child in self.children:
            if not gtkutils.widget_visible(child):
                continue

            left, right, top, bottom = self.get_effective_child_attach(child)
            child.show_submenu_indicator = True
            c_width, c_height = child.size_request()
            toggle_size = child.toggle_size_request()

            max_toggle_size = max(max_toggle_size, toggle_size)
            max_accel_width = max(max_accel_width, child.accelerator_width)

            part = c_width / (right - left)
            requisition.width = max(requisition.width, part)

            part = max(c_height, toggle_size) / (bottom - top)
            self.heights[top] = max(self.heights[top], part)

        for i in range(n_rows):
            requisition.height += self.heights[i]

        requisition.width += max_toggle_size + max_accel_width
        requisition.width *= self.get_n_columns()

        vertical_padding = self.style_get_property('vertical-padding')
        horizontal_padding = self.style_get_property('horizontal-padding')

        requisition.width += (self.border_width + horizontal_padding +
                              self.style.xthickness) * 2
        requisition.height += (self.border_width + vertical_padding +
                               self.style.ythickness) * 2

        self.toggle_size = max_toggle_size

    def do_size_allocate(self, allocation):
        """Overrides widget_class->size_allocate"""
        self.allocation = allocation
        cr_width, cr_height = self.get_child_requisition()

        vertical_padding = self.style_get_property('vertical-padding')
        horizontal_padding = self.style_get_property('horizontal-padding')

        x = self.border_width + self.style.xthickness + horizontal_padding
        y = self.border_width + self.style.ythickness + vertical_padding

        width = max(1, allocation.width - x * 2)
        height = max(1, allocation.height - y * 2)

        cr_width -= x * 2
        cr_height -= y * 2

        if gtkutils.widget_realized(self):
            self.window.move_resize(allocation.x, allocation.y,
                                    allocation.width, allocation.height)
            self.view_window.move_resize(x, y, width, height)


        if self.children:

            n_columns = self.get_n_columns()
            n_rows = self.get_n_rows()

            base_width = width / n_columns

            child_allocation = gtk.gdk.Rectangle()

            for child in self.children:
                if gtkutils.widget_visible(child):
                    (left, right,
                     top, bottom) = self.get_effective_child_attach(child)

                    if self.get_direction() == gtk.TEXT_DIR_RTL:
                        left, right = n_columns - right, n_columns - left

                    child_allocation.width = (right - left) * base_width
                    child_allocation.height = 0
                    child_allocation.x = left * base_width
                    child_allocation.y = 0

                    for i in range(bottom):
                        if i < top:
                            child_allocation.y += self.heights[i]
                        else:
                            child_allocation.height += self.heights[i]

                    child.toggle_size_allocate(self.toggle_size)
                    child.size_allocate(child_allocation)
                    child.queue_draw()

            # resize the item window
            if gtkutils.widget_realized(self):
                height = 0
                for i in range(n_rows):
                    height += self.heights[i]
                width = n_columns * base_width

                self.bin_window.resize(width, height)

    def paint(self, event):
        """Helper metod for the expose event handler"""
        vertical_padding = self.style_get_property('vertical-padding')
        horizontal_padding = self.style_get_property('horizontal-padding')

        border_x = (self.border_width + self.style.xthickness +
                    horizontal_padding)
        border_y = (self.border_width + self.style.ythickness +
                    vertical_padding)
        width, height = self.window.get_size()

        if event.window == self.window:
            self.style.paint_box(self.window, gtk.STATE_NORMAL, gtk.SHADOW_OUT,
                                 None, self, 'menu', 0, 0, -1, -1)

        elif event.window == self.bin_window:
            self.style.paint_box(self.bin_window, gtk.STATE_NORMAL,
                                 gtk.SHADOW_OUT, None, self, 'menu',
                                 - border_x, - border_y, width, height)

    def do_expose_event(self, event):
        """Overrides widget_class->expose_event"""
        if gtkutils.widget_drawable(self):
            self.paint(event)

            MenuShell.do_expose_event(self, event)

        return False

    def do_button_press_event(self, event):
        """Overrides widget_class->button_press_event"""
        # As we do some event propagation in MenuShell event handlers we
        # need to make sure this is the right event type
        if event.type != gtk.gdk.BUTTON_PRESS:
            return False

        return MenuShell.do_button_press_event(self, event)

    def do_button_release_event(self, event):
        """Overrides widget_class->button_release_event"""
        # As we do some event propagation in MenuShell event handlers we
        # need to make sure this is the right event type
        if event.type != gtk.gdk.BUTTON_RELEASE:
            return False

        return MenuShell.do_button_release_event(self, event)

    def do_motion_notify_event(self, event):
        """Overrides widget_class->motion_notify_event"""
        menu_item = gtkutils.get_event_widget(event)
        if (not menu_item or
            not isinstance(menu_item, MenuItem) or
            not isinstance(menu_item.parent, Menu)):
            return False

        menu_shell = menu_item.parent

        if definitely_within_item(menu_item, event.x, event.y):
            menu_shell.activate_time = 0

        need_enter = menu_shell.ignore_enter

        if not menu_item.is_selectable():
            menu_shell.deselect()
            return False

        if need_enter:
            menu_shell.ignore_enter = False

            width, height = event.window.get_size()

            if 0 <= event.x < width and 0 <= event.y < height:
                send_event = gtk.gdk.Event(gtk.gdk.ENTER_NOTIFY)

                send_event.window = event.window
                send_event.time = event.time
                send_event.send_event = True
                send_event.x_root = event.x_root
                send_event.y_root = event.y_root
                send_event.x = event.x
                send_event.y = event.y

                return self.event(send_event)

        return False

    def do_enter_notify_event(self, event):
        """Overrides widget_class->enter_notify_event"""
        menu_item = gtkutils.get_event_widget(event)
        if menu_item and isinstance(menu_item, MenuItem):
            menu = menu_item.parent

            if menu and isinstance(menu, Menu):
                if menu.seen_item_enter:
                    menu.activate_time = 0
                elif (event.detail != gtk.gdk.NOTIFY_NONLINEAR and
                      event.detail != gtk.gdk.NOTIFY_NONLINEAR_VIRTUAL):
                    if definitely_within_item(menu_item, event.x, event.y):
                        menu.activate_time = 0

                menu.seen_item_enter = True

        return MenuShell.do_enter_notify_event(self, event)

    def do_leave_notify_event(self, event):
        """Overrides widget_class->leave_notify_event"""
        menu_item = gtkutils.get_event_widget(event)

        if not menu_item or  not isinstance(menu_item, MenuItem):
            return True

        if (self.active_menu_item is not None and
            menu_item.submenu is not None and
            menu_item.submenu_placement == gtk.LEFT_RIGHT):
            if menu_item.submenu.active:
                return True
            elif menu_item == self.active_menu_item:
                self.deselect()
                return True

        return MenuShell.do_leave_notify_event(self, event)

    def deactivate(self):
        """Deactivates the menu hiding it and its parents"""
        parent = self.parent_menu_shell

        self.activate_time = 0
        self.popdown()

        if parent:
            parent.deactivate()

    def position(self):
        """Moves our toplevel window to the right position on screen"""
        screen = self.get_screen()

        pointer_screen, x, y, mask = screen.get_display().get_pointer()

        req_width, req_height = self.size_request()

        if pointer_screen != screen:
            x = max(0, (screen.get_width() - req_width) / 2)
            y = max(0, (screen.get_height() - req_height) / 2)

        self.monitor_num = screen.get_monitor_at_point(x, y)

        if not gtkutils.widget_visible(self.toplevel):
            self.toplevel.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_MENU)

        if self.position_func:
            x, y, push_in = self.position_func(self, False,
                                               self.position_func_data)

            if self.monitor_num < 0:
                self.monitor_num = screen.get_monitor_at_point(x, y)

            monitor = screen.get_monitor_geometry(self.monitor_num)
        else:
            xthickness = self.style.xthickness
            ythickness = self.style.ythickness
            rtl = self.get_direction() == gtk.TEXT_DIR_RTL

            monitor = screen.get_monitor_geometry(self.monitor_num)

            space_left = x - monitor.x
            space_right = monitor.x + monitor.width - x - 1
            space_above = y - monitor.y
            space_below = monitor.y + monitor.height - y - 1

            needed_width = req_width - xthickness

            if needed_width <= space_left or needed_width <= space_right:
                if (rtl and needed_width <= space_left or
                    not rtl and needed_width > space_right):
                    x = x + xthickness - req_width + 1
                else:
                    x = x - xthickness

            elif req_width <= monitor.width:
                if space_left > space_right:
                    x = monitor.x
                else:
                    x = monitor.x + monitor.width - req_width
            else:
                if rtl:
                    x = monitor.x + monitor.width - req_width
                else:
                    x = monitor.x

            needed_height = req_height - ythickness

            if needed_height <= space_above or needed_height <= space_below:
                if needed_height <= space_below:
                    y = y - ythickness
                else:
                    y = y + ythickness - req_height + 1

                y = gtkutils.clamp(y, monitor.y,
                                   monitor.y + monitor.height - req_height)

            elif needed_height > space_below and needed_height > space_above:
                if space_below >= space_above:
                    y = monitor.y + monitor.height - req_height
                else:
                    y = monitor.y

            else:
                y = monitor.y

        x = gtkutils.clamp(x, monitor.x,
                           max(monitor.x,
                               monitor.x + monitor.width - req_width))

        if self.active:
            self.have_position = True
            self.x = x
            self.y = y

        if y + req_height > monitor.y + monitor.height:
            req_height = (monitor.y + monitor.height) - y

        if y < monitor.y:
            req_height -= monitor.y - y
            y = monitor.y

        self.toplevel.move(x, y)

    def reparent(self, new_parent, unrealize):
        """Change our parent"""
        if unrealize:
            self.parent.remove(self)
            new_parent.add(self)
        else:
            MenuShell.reparent(self, new_parent)

    def show_all(self):
        """Shows all widgets except ourselves"""
        self.foreach(gtk.Widget.show_all)

    def hide_all(self):
        """Hides all widgets except ourselves"""
        self.foreach(gtk.Widget.hide_all)

    def attach(self, child,
               left_attach, right_attach, top_attach, bottom_attach):
        """Attaches the child with the given table attach information"""
        if child.parent is None or child.parent == self:
            return
        if left_attach < right_attach:
            return
        if top_attach < bottom_attach:
            return

        a_info = get_attach_info(child)
        a_info.left_attach = left_attach
        a_info.right_attach = right_attach
        a_info.top_attach = top_attach
        a_info.bottom_attach = top_attach

        if not child.parent:
            self.append(child) # this call child.set_parent(self)

        self.queue_resize()

gobject.type_register(Menu)
desc = 'Extra space at the top and bottom of the menu'
gtk.widget_class_install_style_property(Menu,
                                        ('vertical-padding',
                                         gobject.TYPE_INT,
                                         'Vertical Padding',
                                         desc,
                                         0, 100, 1,
                                         gobject.PARAM_READABLE))
desc = 'Extra space at the left and right of the menu'
gtk.widget_class_install_style_property(Menu,
                                        ('horizontal-padding',
                                         gobject.TYPE_INT,
                                         'Horizontal Padding',
                                         desc,
                                         0, 100, 0,
                                         gobject.PARAM_READABLE))
desc = 'When the menu is a submenu, position it this number of pixels offset vertically'
gtk.widget_class_install_style_property(Menu,
                                        ('vertical-offset',
                                         gobject.TYPE_INT,
                                         'Vertical Offset',
                                         desc,
                                         -100, 100, 0,
                                         gobject.PARAM_READABLE))
desc = 'When the menu is a submenu, position it this number of pixels offset horizontally'
gtk.widget_class_install_style_property(Menu,
                                        ('horizontal-offset',
                                         gobject.TYPE_INT,
                                         'Horizontal Offset',
                                         desc,
                                         -100, 100, -2,
                                         gobject.PARAM_READABLE))

# Utility clases and functions for the Menu widget
class _MenuAttachData(object):
    """Internal class used as a data container for menu attachments"""
    attach_widget = None
    detacher = None

class AttachInfo(object):
    """Class holding the information about a widget position inside a menu"""
    def __init__(self):
        """Dummy constructor"""
        self.left_attach = -1
        self.right_attach = -1
        self.top_attach = -1
        self.bottom_attach = -1
        self.effective_left_attach = 0
        self.effective_right_attach = 0
        self.effective_top_attach = 0
        self.effective_bottom_attach = 0

    def is_grid_attached(self):
        """True is the attachment is set after initialization"""
        return (self.left_attach >= 0 and self.right_attach >= 0 and
                self.top_attach >= 0 and self.bottom_attach >= 0)

def get_attach_info(child):
    """Returns the attachment information for a child"""
    a_info = child.get_data(ATTACH_INFO_KEY)
    if not a_info:
        a_info = AttachInfo()
        child.set_data(ATTACH_INFO_KEY, a_info)
    return a_info

def menu_window_event(window, event, menu):
    """Event handler for the GtkWindow of a Menu"""
    handled = False

    if event.type == gtk.gdk.KEY_PRESS or event.type == gtk.gdk.KEY_RELEASE:
        handled = menu.event(event)

    return handled

def menu_window_size_request(window, requisition, menu):
    """Size request handler for the GtkWindow of a Menu"""
    if menu.have_position:
        screen = window.get_screen()
        monitor = screen.get_monitor_geometry(menu.monitor_num)

        if menu.y + requisition.height > monitor.y + monitor.height:
            requisition.height = monitor.y + monitor.height - menu.y

        if menu.y < monitor.y:
            requisition.height -= monitor.y - menu.y

def check_threshold(start_x, start_y, x, y):
    """Helper function for definetely_within_item"""
    return abs(start_x - x) > 8 or abs(start_y - y) > 8

def definitely_within_item(menu_item, x, y):
    """True if the menu_item is point (x, y) is inside the menu_item"""
    width, height = menu_item.event_window.get_size()
    return (check_threshold(0, 0, x, y) and
            check_threshold(width - 1, 0, x, y) and
            check_threshold(width - 1, height - 1, x, y) and
            check_threshold(0, height - 1, x, y))

#
# MenuItem widget
#

class MenuItem(gtk.Item):
    """Editable MenuItem

    You can click on the menuitem label and an entry will replace it allowing
    you to edit the label
    """
    __gsignals__ = {
        'activate': (gobject.SIGNAL_RUN_FIRST | gobject.SIGNAL_ACTION,
                     gobject.TYPE_NONE, tuple()),
        }

    def __init__(self, text, gadget=None, action_group=None):
        """Constructor"""
        gtk.Item.__init__(self)

        self.set_flags(self.flags() | gtk.NO_WINDOW)

        self.event_window = None
        self.submenu = None
        self.toggle_size = 0
        self.show_submenu_indicator = False
        if self.get_direction() == gtk.TEXT_DIR_RTL:
            self.submenu_direction = gtk.DIRECTION_LEFT
        else:
            self.submenu_direction = gtk.DIRECTION_RIGHT
        self.submenu_placement = gtk.TOP_BOTTOM

        self.accelerator_width = 0 # the Menu class uses this

        self.image = None

        self.label = gtk.Label(text)
        self.label.set_property('xalign', 0.0)

        # This entry is used in edit mode
        self.entry = gtk.Entry()
        self.entry.set_has_frame(False)
        self.entry.connect('activate', self._on_entry__activate)
        self.entry.connect('key-press-event', self._on_entry__key_press_event)

        # By default we are not in edit mode
        self.editing = False
        self.add(self.label)

        # Gazpacho stuff
        self.gadget = gadget
        self.action_group = action_group
        self.action = None

    def to_xml(self, document):
        """Returns a XML node with the representation of this menuitem"""
        if self.action:
            element = document.createElement('menuitem')
            element.setAttribute('name', self.action.name)
            element.setAttribute('action', self.action.name)
            return element

    def set_action(self, action):
        """Set the action updating the image and other internal widgets"""
        self.action = action
        self.label.set_text(action.label)
        if action.stock_id:
            image = gtk.image_new_from_stock(action.stock_id,
                                             gtk.ICON_SIZE_MENU)
            image.show()
            self.set_image(image)

    def do_destroy(self):
        """Overrides widget_class->destroy"""
        if self.submenu:
            self.submenu.destroy()

        gtk.Item.do_destroy(self)

    def set_submenu(self, submenu):
        """Set the submenu associated with this item"""
        if self.submenu != submenu:
            self.remove_submenu()
            self.submenu = submenu

            if submenu:
                submenu.attach_to_widget(self, detacher)

            if self.parent:
                self.queue_resize()

    def remove_submenu(self):
        """Removes the submenu"""
        if self.submenu:
            self.submenu.detach()

    def set_placement(self, placement):
        """Set the submenu placement"""
        self.submenu_placement = placement

    def set_image(self, image):
        """Sets the small icon to the left of the label in LTR"""
        if self.image != image:
            if self.image:
                self.image.unparent()
            self.image = image
            image.set_parent(self)
            self.queue_resize()

    def activate(self):
        """Emits the activate signal"""
        self.emit('activate')

    def do_activate(self):
        """Activate default handler, which start editing"""
        self.start_editing()

    def start_editing(self):
        """Replaces the label by the entry and grab the focus to let the
        user change the label"""
        self.editing = True

        self.entry.show()
        self.label.hide()

        self.entry.set_text(self.label.get_text())

        self.remove(self.label)
        self.add(self.entry)

        self.entry.grab_add()
        self.entry.grab_focus()

        self.queue_draw()

    def end_editing(self, cancel=False):
        """Finish editing by updating the label and replacing the entry with
        the label. If cancel is True, it just replace the entry by the label.

        It also updates (or creates) the gazpacho action associated
        """
        # First we update our internal state
        self.editing = False

        self.entry.grab_remove()

        self.label.show()
        self.entry.hide()

        if not cancel:
            self.label.set_text(self.entry.get_text())

        self.remove(self.entry)
        self.add(self.label)

        if isinstance(self.parent, MenuBar):
            self.parent.active = False
        self.queue_draw()

        if not self.gadget or cancel:
            return

        # Then, we update gazpacho state
        project = self.gadget.project

        label = self.label.get_text()
        name = self._label_to_name(label)

        if self.action:
            # update the associated action
            values = {
                'name' : name,
                'label': label,
                'short_label': self.action.short_label,
                'is_important': self.action.is_important,
                'stock_id': self.action.stock_id,
                'tooltip': self.action.tooltip,
                'accelerator': self.action.accelerator,
                'callback': self.action.callback,
                }
            cmd = CommandEditAction(self.action, values, project)
            command_manager.execute(cmd, project)
        else:
            # create the associated action
            new_gact = GAction(self.action_group,
                               name,  # name
                               label, # label
                               None,  # short label
                               False, # is important?
                               None,  # stock id
                               None,  # tooltip
                               None,  # accelerator
                               None)  # callback
            self.action = new_gact
            cmd = CommandAddRemoveAction(self.action_group, new_gact, True)
            command_manager.execute(cmd, project)

            # create the UI definition
            menu_bar = self.get_menu_bar()
            menu_bar.update_ui()

    def _label_to_name(self, label):
        """Creates a unique action name from its label"""
        name = label.replace(' ', '_')
        name = name.lower()
        basic_name = name
        count = 2
        while name in self.action_group.get_action_names():
            name = '%s-%d' % (basic_name, count)
            count += 1
        return name

    def _on_entry__activate(self, entry):
        """When the user clicks enter we finish editing"""
        self.end_editing()

    def _on_entry__key_press_event(self, entry, event):
        if event.keyval == gtk.keysyms.Escape:
            self.end_editing(cancel=True)
            if not self.action:
                self.parent.remove(self)
            return True

        return False

    def _get_minimum_width(self):
        """Minimum width based on 7 characters and the image"""
        context = self.get_pango_context()
        metrics = context.get_metrics(self.style.font_desc,
                                      context.get_language())
        height = metrics.get_ascent() + metrics.get_descent()

        ret = pango.PIXELS(7 * height)

        if self.image:
            width, height = self.image.size_request()
            ret = max(ret, height)

        return ret

    def _get_pack_directions(self):
        """Returns packs directions"""
        if isinstance(self.parent, MenuBar):
            pack_dir = self.parent.get_pack_direction()
            child_pack_dir = self.parent.get_child_pack_direction()
        else:
            pack_dir = PACK_DIRECTION_LTR
            child_pack_dir = PACK_DIRECTION_LTR
        return (pack_dir, child_pack_dir)

    def do_size_request(self, requisition):
        """Overrides widget_class->size_request"""
        horizontal_padding = self.style_get_property('horizontal-padding')

        pack_dir, child_pack_dir = self._get_pack_directions()

        requisition.width = self.border_width + self.style.xthickness * 2
        requisition.height = self.border_width + self.style.ythickness * 2

        if ((pack_dir == PACK_DIRECTION_LTR or
            pack_dir == PACK_DIRECTION_RTL) and
            (child_pack_dir == PACK_DIRECTION_LTR or
             child_pack_dir == PACK_DIRECTION_RTL)):
            requisition.width += 2 * horizontal_padding
        elif ((pack_dir == PACK_DIRECTION_TTB or
               pack_dir == PACK_DIRECTION_BTT) and
              (child_pack_dir == PACK_DIRECTION_TTB or
               child_pack_dir == PACK_DIRECTION_BTT)):
            requisition.height += 2 * horizontal_padding

        if self.child and gtkutils.widget_visible(self.child):
            child_width, child_height = self.child.size_request()
            requisition.width += child_width
            requisition.height += child_height

            if self.submenu and self.show_submenu_indicator:
                arrow_spacing = self.style_get_property('arrow-spacing')

                requisition.width += child_height
                requisition.width += arrow_spacing
                requisition.width = max(requisition.width,
                                        self._get_minimum_width())

        else: # separator item
            requisition.height += 4

    def do_size_allocate(self, allocation):
        """Overrides widget_class->size_allocate"""
        direction = self.get_direction()

        pack_dir, child_pack_dir = self._get_pack_directions()

        self.allocation = allocation

        if self.child:
            horizontal_padding = self.style_get_property('horizontal_padding')
            child_allocation = gtk.gdk.Rectangle()
            child_allocation.x = self.border_width + self.style.xthickness
            child_allocation.y = self.border_width + self.style.ythickness

            if ((pack_dir == PACK_DIRECTION_LTR or
                 pack_dir == PACK_DIRECTION_RTL) and
                (child_pack_dir == PACK_DIRECTION_LTR or
                 child_pack_dir == PACK_DIRECTION_RTL)):
                child_allocation.x += horizontal_padding
            elif ((pack_dir == PACK_DIRECTION_TTB or
                   pack_dir == PACK_DIRECTION_BTT) and
                  (child_pack_dir == PACK_DIRECTION_TTB or
                   child_pack_dir == PACK_DIRECTION_BTT)):
                child_allocation.y += horizontal_padding

            temp_width = allocation.width - child_allocation.x * 2
            child_allocation.width = max(1, temp_width)

            temp_height = allocation.height - child_allocation.y * 2
            child_allocation.height = max(1, temp_height)

            if (child_pack_dir == PACK_DIRECTION_LTR or
                child_pack_dir == PACK_DIRECTION_RTL):
                if ((direction == gtk.TEXT_DIR_LTR) ==
                    (child_pack_dir != PACK_DIRECTION_RTL)):
                    child_allocation.x += self.toggle_size
                child_allocation.width -= self.toggle_size
            else:
                if ((direction == gtk.TEXT_DIR_LTR) ==
                    (child_pack_dir != PACK_DIRECTION_BTT)):
                    child_allocation.y += self.toggle_size
                child_allocation.height -= self.toggle_size

            child_allocation.x += self.allocation.x
            child_allocation.y += self.allocation.y

            c_req_width, c_req_height = self.child.get_child_requisition()

            if self.submenu and self.show_submenu_indicator:
                if direction == gtk.TEXT_DIR_RTL:
                    child_allocation.x += c_req_width
                child_allocation.width -= c_req_height

            if child_allocation.width < 1:
                child_allocation.width = 1

            self.child.size_allocate(child_allocation)

            if self.image:
                width, height = self.image.get_child_requisition()
                image_allocation = gtk.gdk.Rectangle()
                image_allocation.x = self.border_width + self.style.xthickness
                image_allocation.y = self.border_width + self.style.ythickness
                y = (self.allocation.height - height) / 2
                image_allocation.y += max(y, 0) + self.allocation.y
                image_allocation.width = width
                image_allocation.height = height
                self.image.size_allocate(image_allocation)

        if gtkutils.widget_realized(self):
            self.event_window.move_resize(allocation.x, allocation.y,
                                          allocation.width, allocation.height)

        if self.submenu:
            self.submenu.reposition()

    def do_realize(self):
        """Overrides widget_class->realize to create event_window"""
        self.set_flags(self.flags() | gtk.REALIZED)
        self.window = self.get_parent_window()

        self.event_window = gtk.gdk.Window(
            self.window,
            window_type=gtk.gdk.WINDOW_CHILD,
            x=self.allocation.x,
            y=self.allocation.y,
            width=self.allocation.width,
            height=self.allocation.height,
            wclass=gtk.gdk.INPUT_ONLY,
            event_mask=(self.get_events() |
                        gtk.gdk.BUTTON_PRESS_MASK |
                        gtk.gdk.BUTTON_RELEASE_MASK |
                        gtk.gdk.ENTER_NOTIFY_MASK |
                        gtk.gdk.LEAVE_NOTIFY_MASK |
                        gtk.gdk.POINTER_MOTION_MASK))

        self.event_window.set_user_data(self)

        self.set_style(self.style.attach(self.window))

    def do_unrealize(self):
        """Overrides widget_class->unrealize to destroy event_window"""
        self.event_window.set_user_data(None)
        self.event_window.destroy()
        self.event_window = None
        gtk.Item.do_unrealize(self)

    def do_map(self):
        """Overrides widget_class->map to map the event_window"""
        gtk.Item.do_map(self)
        self.event_window.show()

    def do_unmap(self):
        """Overrides widget_class->unmap to unmap the event_window"""
        self.event_window.hide()
        gtk.Item.do_unmap(self)

    def paint(self, area):
        """Helper method for the expose event handler"""
        if gtkutils.widget_drawable(self):
            x = self.allocation.x + self.border_width
            y = self.allocation.y + self.border_width
            width = self.allocation.width - self.border_width * 2
            height = self.allocation.height - self.border_width * 2

            if self.state == gtk.STATE_PRELIGHT and self.child:
                sel_shdw_type = self.style_get_property('selected-shadow-type')
                self.style.paint_box(self.window, gtk.STATE_PRELIGHT,
                                     sel_shdw_type, area, self, 'menuitem',
                                     x, y, width, height)

            if self.submenu and self.show_submenu_indicator:
                direction = self.get_direction()

                hor_padding = self.style_get_property('horizontal-padding')

                context = self.child.get_pango_context()
                metrics = context.get_metrics(self.child.style.font_desc,
                                              context.get_language())
                ascent = metrics.get_ascent()
                descent = metrics.get_descent()

                arrow_size = (pango.PIXELS(ascent + descent) -
                              2 * self.style.ythickness)
                arrow_extent = int(arrow_size * 0.8)

                shadow_type = gtk.SHADOW_OUT
                if self.state == gtk.STATE_PRELIGHT:
                    shadow_type = gtk.SHADOW_IN

                if direction == gtk.TEXT_DIR_LTR:
                    arrow_x = x + width - hor_padding - arrow_extent
                    arrow_type = gtk.ARROW_RIGHT
                else:
                    arrow_x = x + hor_padding
                    arrow_type = gtk.ARROW_LEFT

                arrow_y = y + (height - arrow_extent) / 2

                self.style.paint_arrow(self.window, self.state, shadow_type,
                                       area, self, 'menuitem', arrow_type,
                                       True, arrow_x, arrow_y,
                                       arrow_extent, arrow_extent)
            elif not self.child:
                hor_padding = self.style_get_property('horizontal-padding')

                sx1 = (self.allocation.x + hor_padding +
                       self.style.xthickness)
                sx2 = (self.allocation.x + self.allocation.width -
                       hor_padding - self.style.xthickness - 1)
                sep_y = self.allocation.y + (self.allocation.height -
                                             self.style.ythickness) / 2
                self.style.paint_hline(self.window, gtk.STATE_NORMAL,
                                       area, self, 'menuitem',
                                       sx1, sx2, sep_y)

    def do_expose_event(self, event):
        """Overrides widget_class->expose"""
        if gtkutils.widget_drawable(self):
            self.paint(event.area)

            gtk.Item.do_expose_event(self, event)

        return False

    def do_select(self):
        """Select signal handler. Pop up the submenu"""
        if self.submenu and not gtkutils.widget_mapped(self.submenu):
            self._popup_submenu()

        self.set_state(gtk.STATE_PRELIGHT)
        self.queue_draw()

    def do_deselect(self):
        """Deselect signal handler. Pop down the submenu"""
        if self.submenu:
            self.submenu.popdown()

        self.set_state(gtk.STATE_NORMAL)
        self.queue_draw()

    def toggle_size_request(self):
        """Compute the size request for the toggle area"""
        if self.image:
            width, height = self.image.size_request()
            return width
        else:
            return 0

    def toggle_size_allocate(self, allocation):
        """Allocate the size for the toggle area"""
        self.toggle_size = allocation

    def _popup_submenu(self):
        """Shows the submenu"""
        if gtkutils.widget_is_sensitive(self.submenu):
            self.submenu.take_focus = self.parent.take_focus

            self.submenu.popup(self.parent, self, menu_item_position_menu,
                               self, self.parent.button, 0)

    def do_show_all(self):
        """Show the children including the submenu"""
        if self.submenu:
            self.submenu.show_all()

        if self.child:
            self.child.show_all()

        if self.image:
            self.image.show_all()

        self.show()

    def do_hide_all(self):
        """Hide the children including the submenu"""
        self.hide()

        if self.image:
            self.image.hide_all()

        if self.child:
            self.child.hide_all()

        if self.submenu:
            self.submenu.hide_all()

    def do_forall(self, include_internals, callback, data):
        """Iterates through the label and image"""
        if self.child:
            callback(self.child, data)

        if self.image:
            callback(self.image, data)

    def do_remove(self, child):
        """Remove the label or the image"""
        if self.image == child:
            was_visible = gtkutils.widget_visible(child)
            child.unparent()
            self.image = None
            if gtkutils.widget_visible(self) and was_visible:
                self.queue_resize()
        else:
            gtk.Item.do_remove(self, child)

    def is_selectable(self):
        """True if the item can be selected"""
        if (not self.child or
            isinstance(self, SeparatorMenuItem) or
            not gtkutils.widget_is_sensitive(self) or
            not gtkutils.widget_visible(self)):
            return False

        return True

    def is_first(self):
        """True if the item is the first child of its parent"""
        if self.parent:
            first = self.parent.children[0]
            if first == self:
                return True
        return False

    def is_last(self):
        """True if the item is the last child of its parent"""
        if self.parent:
            last = self.parent.children[-2] # last element is the placeholder
            if last == self:
                return True
        return False

    def is_new_group(self):
        """True is there is a Separator just before this item"""
        if self.parent:
            i = self.parent.children.index(self)
            if i > 0:
                prev = self.parent.children[i - 1]
                if isinstance(prev, SeparatorMenuItem):
                    return True
        return False

    def get_menu_bar(self):
        """Return the top menubar this menuitem belongs to"""
        parent = self.parent
        if isinstance(parent, MenuBar):
            return parent
        elif isinstance(parent, Menu):
            menuitem = parent.get_attach_widget()
            return menuitem.get_menu_bar()

gobject.type_register(MenuItem)
gtk.widget_class_install_style_property(MenuItem,
                                        ('selected-shadow-type',
                                         gtk.ShadowType,
                                         'Selected Shadow Type',
                                         'Shadow type when item is selected',
                                         gtk.SHADOW_NONE,
                                         gobject.PARAM_READABLE))

gtk.widget_class_install_style_property(MenuItem,
                                        ('horizontal-padding',
                                         gobject.TYPE_INT,
                                         'horizontal-padding',
                                         'Padding to left and right',
                                         0, 100, 3,
                                         gobject.PARAM_READABLE))
gtk.widget_class_install_style_property(MenuItem,
                                        ('arrow-spacing',
                                         gobject.TYPE_INT,
                                         'Arrow Spacing',
                                         'Space between label and arrow',
                                         0, 100, 10,
                                         gobject.PARAM_READABLE))

class SeparatorMenuItem(MenuItem):
    """Item for separator entries"""
    def __init__(self):
        """Basic constructor"""
        gtk.Item.__init__(self) # note we don't call MenuItem __init__

        self.set_flags(self.flags() | gtk.NO_WINDOW)

        self.submenu = None
        self.toggle_size = 0
        self.show_submenu_indicator = False
        if self.get_direction() == gtk.TEXT_DIR_RTL:
            self.submenu_direction = gtk.DIRECTION_LEFT
        else:
            self.submenu_direction = gtk.DIRECTION_RIGHT
        self.submenu_placement = gtk.TOP_BOTTOM

        self.accelerator_width = 0 # the Menu class uses this

        self.image = None

        self.label = None
        self.entry = None
        self.editing = False
        self.action = None

    def to_xml(self, document):
        """Returns a xml node representing this item"""
        element = document.createElement('separator')
        return element

gobject.type_register(SeparatorMenuItem)

def get_offsets(menu):
    """Helper function for menu_item_position_menu"""
    horizontal_offset = menu.style_get_property('horizontal-offset')
    vertical_offset = menu.style_get_property('vertical-offset')
    horizontal_padding = menu.style_get_property('horizontal-padding')
    vertical_padding = menu.style_get_property('vertical_padding')

    vertical_offset -= menu.style.ythickness
    vertical_offset -= vertical_padding
    horizontal_offset += horizontal_padding

    return (horizontal_offset, vertical_offset)

def menu_item_position_menu(menu_widget, push_in, menu_item):
    """Computes the position where the menu should be positioned given the
    menu_item"""
    if push_in:
        push_in = False

    direction = menu_item.get_direction()
    twidth, theight = menu_widget.size_request()

    screen = menu_widget.get_screen()

    monitor_num = screen.get_monitor_at_window(menu_item.event_window)
    if monitor_num < 0:
        monitor_num = 0

    monitor = screen.get_monitor_geometry(monitor_num)

    tx, ty = menu_item.window.get_origin()

    tx += menu_item.allocation.x
    ty += menu_item.allocation.y

    horizontal_offset, vertical_offset = get_offsets(menu_widget)

    if menu_item.submenu_placement == gtk.TOP_BOTTOM:
        if direction == gtk.TEXT_DIR_LTR:
            menu_item.submenu_direction = gtk.DIRECTION_RIGHT
        else:
            menu_item.submenu_direction = gtk.DIRECTION_LEFT
            tx += menu_item.allocation.width - twidth

        if ((ty + menu_item.allocation.height + theight) <=
            monitor.y + monitor.height):
            ty += menu_item.allocation.height
        elif ((ty - theight) >= monitor.y):
            ty -= theight
        elif (monitor.y + monitor.height -
              (ty + menu_item.allocation.height) > ty):
            ty += menu_item.allocation.height
        else:
            ty -= theight

    elif menu_item.submenu_placement == gtk.LEFT_RIGHT:
        if isinstance(menu_item.parent, Menu):
            parent_menu_item = menu_item.parent.parent_menu_item
        else:
            parent_menu_item = None

        parent_xthickness = menu_item.parent.style.xthickness

        if parent_menu_item:
            menu_item.submenu_direction = parent_menu_item.submenu_direction
        else:
            if direction == gtk.TEXT_DIR_LTR:
                menu_item.submenu_direction = gtk.DIRECTION_RIGHT
            else:
                menu_item.submenu_direction = gtk.DIRECTION_LEFT

        new_direction = None
        if menu_item.submenu_direction == gtk.DIRECTION_LEFT:
            if ((tx - twidth - parent_xthickness - horizontal_offset) >=
                monitor.x):
                tx -= twidth + parent_xthickness + horizontal_offset
            else:
                new_direction = gtk.DIRECTION_RIGHT
                tx += (menu_item.allocation.width + parent_xthickness +
                       horizontal_offset)

        elif menu_item.submenu_direction == gtk.DIRECTION_RIGHT:
            if ((tx + menu_item.allocation.width + parent_xthickness +
                 horizontal_offset +twidth) <= monitor.x + monitor.width):
                tx += (menu_item.allocation.width + parent_xthickness +
                       horizontal_offset)
            else:
                new_direction = gtk.DIRECTION_LEFT
                tx -= twidth + parent_xthickness + horizontal_offset


        if new_direction:
            menu_item.submenu_direction = new_direction

        ty += vertical_offset

        ty = gtkutils.clamp(ty, monitor.y,
                            max(monitor.y,
                                monitor.y + monitor.height - theight))

    x = gtkutils.clamp(tx, monitor.x,
                       max(monitor.x, monitor.x + monitor.width - twidth))
    y = ty

    return (x, y, push_in)

def detacher(menuitem, menu):
    """Function to detach menuitem from menu"""
    if not menuitem.submenu == menu:
        return

    menuitem.submenu = None


class AddMenuItem(MenuItem):
    """MenuItem that can add menus to a MenuBar"""
    def __init__(self, gadget):
        """Creates the menuitem with a 'Click to create menu' label"""
        text = _('Click to create menu')
        label = '<i><span foreground="darkgrey">%s</span></i>' % text
        MenuItem.__init__(self, label)

        self.gadget = gadget

        self.label.set_use_markup(True)

    def do_activate(self):
        """Creates a menuitem and a submenu for it"""
        project = self.gadget.project
        uim = project.uim
        default_action_group = uim.get_action_group('DefaultActions')
        if default_action_group:
            item = MenuItem(_('New Menu'), self.gadget, default_action_group)
            submenu = Menu(self.gadget)
            item.set_submenu(submenu)
            item.show_all()
            self.parent.insert(item, len(self.parent.children) - 1)
            item.activate()
        else:
            error(_('There is no action group to add this action to'))

    def do_select(self):
        """As soon as the item is selected it is activated"""
        self.activate()

    def to_xml(self, document):
        """An AddMenuItem is not serialized so this method returns None"""
        return None

class AddActionItem(MenuItem):
    """MenuItem that can add actions to a Menu"""
    def __init__(self, gadget):
        """Creates the menuitem with a 'Click to create action' label"""
        text = _('Click to create action')
        label = '<i><span foreground="darkgrey">%s</span></i>' % text
        MenuItem.__init__(self, label)

        self.gadget = gadget

        self.label.set_use_markup(True)

    def do_activate(self):
        """Creates a menuitem"""
        project = self.gadget.project
        uim = project.uim
        default_action_group = uim.get_action_group('DefaultActions')
        if default_action_group:
            item = MenuItem(_('New Action'), self.gadget, default_action_group)
            item.show_all()
            self.parent.insert(item, len(self.parent.children) - 1)
            item.activate()
        else:
            error(_('There is no action group to add this action to'))

    def to_xml(self, document):
        """An AddMenuItem is not serialized so this method returns None"""
        return None



import os.path
from notmm.utils.urlmap import RegexURLMap

from django.contrib import admin
admin.autodiscover()

urlpatterns = RegexURLMap()
urlpatterns.add_routes('',
    (r'^$', 'localsite.views.example', {})
    )
urlpatterns.add_routes('',    
    (r'^media/(?P<path>.*)$', 'django.views.static.serve',
        {'document_root': os.path.join(os.path.dirname(__file__), 'media/')})
    )

urlpatterns.include(admin.site.urls, prefix='^admin/')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Test suite for the ``notmm.utils.testlib`` package.
# 
import os
import re
import sys

from notmm.utils.configparse import loadconf, string_getter
from notmm.utils.testlib import *

from test_support import unittest

class TestCollectionTestCase(unittest.TestCase):
    
    app_conf = loadconf('development.ini')
        
    def setUp(self):
        self.loader = TestCollection()

    def test_load_tests_from_names(self):
        
        #XXX inexistent packages should raises CollectionError :)
        self.assertRaises(ImportError, \
            self.loader.loadTestsFromList, ('werkzeug'))

    #def test_get_base_dir(self):
    #    from notmm.utils.testlib.collection2 import get_base_dir
    #    self.assertEqual(get_base_dir('apps/authkit'), 'apps/authkit')

    def test_find_all_unique_test_cases(self):
        # XXX ``TestCollection.findTestCases`` must be able to load
        # test cases from different files while having the same
        # file name...
        method = 'findTestCases'
        self.assertEqual(hasattr(self.loader, method), True)
        testSuite = self.loader.loadTestsFromNames(
            'apps/blogengine'
            )
        self.assertEqual(isinstance(testSuite, \
            unittest.TestSuite), True)



from django import forms
from django.contrib.auth import login, authenticate
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _, ugettext
from satchmo_store.accounts.mail import send_welcome_email
from livesettings import config_value
from satchmo_store.contact.forms import ContactInfoForm
from satchmo_store.contact.models import AddressBook, PhoneNumber, Contact, ContactRole
from l10n.models import Country
from satchmo_utils.unique_id import generate_id
from signals_ahoy.signals import form_init, form_initialdata

import logging
import signals

log = logging.getLogger('accounts.forms')

class EmailAuthenticationForm(AuthenticationForm):
    """Authentication form with a longer username field."""
    
    def __init__(self, *args, **kwargs):
        
        super(EmailAuthenticationForm, self).__init__(*args, **kwargs)
        username = self.fields['username']
        username.max_length = 75
        username.widget.attrs['maxlength'] = 75

class RegistrationForm(forms.Form):
    """The basic account registration form."""
    title = forms.CharField(max_length=30, label=_('Title'), required=False)
    email = forms.EmailField(label=_('Email address'),
        max_length=75, required=True)
    password2 = forms.CharField(label=_('Password (again)'),
        max_length=30, widget=forms.PasswordInput(), required=True)
    password1 = forms.CharField(label=_('Password'),
        max_length=30, widget=forms.PasswordInput(), required=True)
    first_name = forms.CharField(label=_('First name'),
        max_length=30, required=True)
    last_name = forms.CharField(label=_('Last name'),
        max_length=30, required=True)
    next = forms.CharField(max_length=100, required=False, widget=forms.HiddenInput())

    def __init__(self, *args, **kwargs):
        contact = kwargs.get('contact', None)
        initial = kwargs.get('initial', {})
        self.contact = contact
        form_initialdata.send(self.__class__,
            form=self,
            contact=contact,
            initial=initial)
        
        kwargs['initial'] = initial
        super(RegistrationForm, self).__init__(*args, **kwargs)
        form_init.send(self.__class__,
            form=self,
            contact=contact)

    newsletter = forms.BooleanField(label=_('Newsletter'),
        widget=forms.CheckboxInput(), required=False)

    def clean_password1(self):
        """Enforce that password and password2 are the same."""
        p1 = self.cleaned_data.get('password1')
        p2 = self.cleaned_data.get('password2')
        if not (p1 and p2 and p1 == p2):
            raise forms.ValidationError(
                ugettext("The two passwords do not match."))

        # note, here is where we'd put some kind of custom
        # validator to enforce "hard" passwords.
        return p1

    def clean_email(self):
        """Prevent account hijacking by disallowing duplicate emails."""
        email = self.cleaned_data.get('email', None)
        if email and User.objects.filter(email__iexact=email).count() > 0:
            raise forms.ValidationError(
                ugettext("That email address is already in use."))

        return email

    def save(self, request=None, **kwargs):
        """Create the contact and user described on the form.  Returns the
        `contact`.
        """
        if self.contact:
            log.debug('skipping save, already done')
        else:
            self.save_contact(request)
        return self.contact

    def save_contact(self, request):
        log.debug("Saving contact")
        data = self.cleaned_data
        password = data['password1']
        email = data['email']
        first_name = data['first_name']
        last_name = data['last_name']
        username = generate_id(first_name, last_name, email)

        verify = (config_value('SHOP', 'ACCOUNT_VERIFICATION') == 'EMAIL')

        if verify:
            from registration.models import RegistrationProfile
            user = RegistrationProfile.objects.create_inactive_user(
                username, password, email, send_email=True)
        else:
            user = User.objects.create_user(username, email, password)

        user.first_name = first_name
        user.last_name = last_name
        user.save()

        # If the user already has a contact, retrieve it.
        # Otherwise, create a new one.
        try:
            contact = Contact.objects.from_request(request, create=False)

        except Contact.DoesNotExist:
            contact = Contact()

        contact.user = user
        contact.first_name = first_name
        contact.last_name = last_name
        contact.email = email
        contact.role = ContactRole.objects.get(pk='Customer')
        contact.title = data.get('title', '')
        contact.save()

        if 'newsletter' not in data:
            subscribed = False
        else:
            subscribed = data['newsletter']

        signals.satchmo_registration.send(self, contact=contact, subscribed=subscribed, data=data)

        if not verify:
            user = authenticate(username=username, password=password)
            login(request, user)
            send_welcome_email(email, first_name, last_name)
            signals.satchmo_registration_verified.send(self, contact=contact)

        self.contact = contact

        return contact

class RegistrationAddressForm(RegistrationForm, ContactInfoForm):
    """Registration form which also requires address information."""
    
    def __init__(self, *args, **kwargs):
        super(RegistrationAddressForm, self).__init__(*args, **kwargs)

    def save(self, request=None, **kwargs):
        contact = self.save_contact(request)
        kwargs['contact'] = contact
        
        log.debug('Saving address for %s', contact)
        self.save_info(**kwargs)
                
        return contact



from django import http
from django.template import RequestContext
from django.template import loader
from django.utils.translation import ugettext as _

ccInfo = (
    #  type, prefix, length
    ( 'VISA', '4', 16),
    ( 'VISA', '4', 13),
    ( 'UKE', '417500', 16),
    ( 'UKE', '4917', 16),
    ( 'UKE', '4913', 16),
    ( 'SWITCH', '4903', 16),
    ( 'SWITCH', '4905', 16),
    ( 'SWITCH', '4911', 16),
    ( 'SWITCH', '4936', 16),
    ( 'SWITCH', '564182', 16),
    ( 'SWITCH', '633110', 16),
    ( 'SWITCH', '6333', 16),
    ( 'SWITCH', '6759', 16),
    ( 'SWITCH', '4903', 18),
    ( 'SWITCH', '4905', 18),
    ( 'SWITCH', '4911', 18),
    ( 'SWITCH', '4936', 18),
    ( 'SWITCH', '564182', 18),
    ( 'SWITCH', '633110', 18),
    ( 'SWITCH', '6333', 18),
    ( 'SWITCH', '6759', 18),
    ( 'SWITCH', '4903', 19),
    ( 'SWITCH', '4905', 19),
    ( 'SWITCH', '4911', 19),
    ( 'SWITCH', '4936', 19),
    ( 'SWITCH', '564182', 19),
    ( 'SWITCH', '633110', 19),
    ( 'SWITCH', '6333', 19),
    ( 'SWITCH', '6759', 19),
    ( 'SOLO', '6334', 16),
    ( 'SOLO', '6767', 16),  
    ( 'SOLO', '6334', 18),
    ( 'SOLO', '6767', 18),      
    ( 'SOLO', '6334', 19),
    ( 'SOLO', '6767', 19),    
    ( 'MAESTRO', '5020', 16),
    ( 'MAESTRO', '6', 16),
    ( 'MC', '51', 16),
    ( 'MC', '52', 16),
    ( 'MC', '53', 16),
    ( 'MC', '54', 16),
    ( 'MC', '55', 16),
    ( 'DISCOVER', '6011', 16),
    ( 'AMEX', '34', 15),
    ( 'AMEX', '37', 15),
    ( 'DC', '300', 14),
    ( 'DC', '301', 14),
    ( 'DC', '302', 14),
    ( 'DC', '303', 14),
    ( 'DC', '304', 14),
    ( 'DC', '305', 14),
    ( 'DC', '36', 14),
    ( 'DC', '38', 14),
    ( 'JCB', '3', 16),
    ( 'JCB', '2131', 15),
    ( 'JCB', '1800', 15),
)

class CreditCard(object):
    def __init__(self, number, cardtype):
        self.card_number = number
        self.card_type = cardtype

    def _verifyMod10(self, number):
        '''Check a credit card number for validity using the mod10 algorithm.'''
        double = 0
        sum = 0
        for i in range(len(number) - 1, -1, -1):
            for c in str((double + 1) * int(number[i])): sum = sum + int(c)
            double = (double + 1) % 2
        return ((sum % 10) == 0)

    def _stripCardNum(self, card):
        '''Return card number with all non-digits stripped.  '''
        import re
        return re.sub(r'[^0-9]', '', self.card_number)

    def verifyCardNumber(self):
        '''Return card type string if legal, None otherwise.
        Check the card number and return a string representing the card type if
        it could be a valid card number.

        RETURNS: (String) Credit card type string if legal.(None) if invalid.
        '''
        s = self._stripCardNum(self.card_number)
        if self._verifyMod10(s):
            return self.card_type
        return None

    def verifyCardTypeandNumber(self):
        card_check_type = self.verifyCardNumber()
        if card_check_type:
            if card_check_type == self.card_type:
                return (True, None)
            else:
                return (False, _("Card type does not match card number."))
        else:
            return (False, _("Invalid credit card number."))

def bad_or_missing(request, msg):
    """
    Return an HTTP 404 response for a date request that cannot possibly exist.
    The 'msg' parameter gives the message for the main panel on the page.
    """
    template = loader.get_template('shop/404.html')
    context = RequestContext(request, {'message': msg})
    return http.HttpResponseNotFound(template.render(context))



#!/usr/bin/env python2.5
"""manage.py script to perform management tasks"""

import sys,os

def setup_environ():
    """Environment sanity checks"""
    required_keys = ('DJANGO_HOME', 'DJANGO_SETTINGS_MODULE')
    for name in required_keys:
        if not os.environ.has_key(name):
            # Raise an error since this is a requirement
            raise EnvironmentError('Please set %r' % name)
        else:
            print 'Found %s .. ok' % repr(name)
    # Add `DJANGO_HOME` to sys.path
    sys.path.append(os.environ['DJANGO_HOME'])

def init_main(argv=None, fallback='settings'):
    """A wrapper for execute_manager"""
    try:
        setup_environ()
        module_name  = os.environ['DJANGO_SETTINGS_MODULE'] or fallback
        settings_module = __import__(module_name, {}, {}, [])
        #settings_module = sys.modules[module_name]

    except (ImportError, EnvironmentError), e:
        raise e
    else:
        from django.core.management import execute_from_command_line
        execute_from_command_line(argv)

if __name__ == "__main__":
    init_main(argv=sys.argv)




from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from livesettings import *
from satchmo_utils import load_module
from satchmo_store.shop import get_satchmo_setting

NEWSLETTER_GROUP = ConfigurationGroup('NEWSLETTER', _('Newsletter Settings'))

NEWSLETTER_ACTIVE = config_register(StringValue(NEWSLETTER_GROUP,
    'MODULE',
    description=_("Active newsletter module"),
    help_text=_("Select a newsletter, save and reload to set any module-specific newsletter settings."),
    default="satchmo_ext.newsletter.simple",
    choices=[('satchmo_ext.newsletter.ignore', _('No newsletter')),
            ('satchmo_ext.newsletter.simple', _('Simple list')),
             ('satchmo_ext.newsletter.mailman', _('Mailman'))]
    ))
    
config_register(StringValue(NEWSLETTER_GROUP,
    'NEWSLETTER_NAME',
    description=_("Name of Newsletter"),
    help_text=_("""Give the exact name that matches the mailing list set up in Mailman."""),
    requires=NEWSLETTER_ACTIVE,
    requiresvalue='satchmo_ext.newsletter.mailman',
    default=""
    ))

config_register(
    StringValue(NEWSLETTER_GROUP,
        'NEWSLETTER_SLUG',
        description=_('Newsletter Slug'),
        help_text=_("The url slug for the newsletter.  Requires server restart if changed."),
        default="newsletter"))

# --- Load any extra newsletter modules. ---
extra_payment = get_satchmo_setting('CUSTOM_NEWSLETTER_MODULES')

for extra in extra_payment:
    try:
        load_module("%s.config" % extra)
    except ImportError:
        log.warn('Could not load payment module configuration: %s' % extra)



#!/usr/bin/env python
import shelve
import feedcache

storage = shelve.open('.feeds_cache')

feedparser = feedcache.Cache(storage)
feed = feedparser.fetch('http://www.linuxinsight.com/rss.xml')
print type(feed)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#from notmm.utils.urlmap import RegexURLMap, url
from django.conf.urls import url
from notmm.utils.template import direct_to_template
from sandbox_sqlalchemy import views

# a custom/generic login view
handleLogin = 'blogengine2.contrib.session.login'
handleLogout = 'blogengine2.contrib.session.logout'

urlpatterns = [
        url(r'^test_with_session/$', views.test_with_session)
]
# Add the authentication views
#urlpatterns.add_routes('',
        # Custom authentication backend wrapper (authkit based)
        # see nomainapp..controllers.auth.LoginController
        #url(r'^session_login/$', handleLogin, \
        #    dict(template_name='auth/login.mako')),
        #url(r'^session_logout/$', handleLogout, \
        #    dict(template_name='auth/logout.mako')),
#)

#urlpatterns.include('sandbox_sqlalchemy.config.extras')

#blogengine api v1
#urlpatterns.include('blogengine.contrib.api_v1.urls', prefix="^blog/")





"""
TieredWeight shipping models
"""
import logging

from datetime import date
from django.db import models
from django.conf import settings
from django.utils.translation import get_language, ugettext_lazy as _
from l10n.models import Country
from shipping.modules.base import BaseShipper

try:
    from decimal import Decimal
except:
    from django.utils._decimal import Decimal


log = logging.getLogger('shipping.TieredWeight')


class TieredWeightException(Exception):
    pass


def _get_cart_weight(cart):
    weight = Decimal('0.0')
    for item in cart.cartitem_set.all():
        if item.is_shippable and item.product.weight:
            weight = weight + (item.product.weight * item.quantity)
    return weight


class Shipper(BaseShipper):
    def __init__(self, carrier):
        self.id = 'tieredweight_%i' % carrier.pk
        self._carrier = carrier
        super(BaseShipper, self).__init__()


    def calculate(self, cart, contact):
        """
        Perform shipping calculations
        """
        self._cost, self._weight = None, None
        self._zone = self._carrier.get_zone(contact._shipping_address().country)

        if self._zone:
            try:
                self._weight = _get_cart_weight(cart)
                self._cost = self._zone.cost(self._weight)
            except TieredWeightException:
                pass

        super(Shipper, self).calculate(cart, contact)


    def __str__(self):
        """
        This is mainly helpful for debugging purposes
        """
        return "TieredWeight_Shipper: %s" % self.id


    def cost(self):
        """
        Calulates the shipping for the order
        """
        assert(self._calculated)
        return self._cost


    def description(self):
        """
        A basic description that will be displayed to the user when selecting their shipping options
        """
        assert(self._calculated)
        return self._zone.description


    def method(self):
        """
        Describes the actual delivery service (Mail, FedEx, DHL, UPS, etc)
        """
        assert(self._calculated)
        return self._zone.method


    def expectedDelivery(self):
        """
        Can be a plain string or complex calculation returning an actual date
        """
        assert(self._calculated)
        return self._zone.delivery


    def valid(self, order=None):
        """
        Check if shipping is valid for country and set zone accordingly. Fallback 
        to default zone if set
        """
        assert(self._calculated)
        # I think its reasonable to assume this shipping method should
        # not be used on an order that doesn't weigh anything.
        if not self._weight or self._weight == Decimal('0.0'):
            return False

        if self._zone and self._cost:
            return True


class Carrier(models.Model):
    name = models.CharField(_('carrier'), max_length=50)
    ordering = models.IntegerField(_('Ordering'), default=0)
    active = models.BooleanField(_('Active'), default=True)
    default_zone = models.ForeignKey('Zone', verbose_name=_('default_zone'), 
        related_name='default', null=True, blank=True)


    def __unicode__(self):
        return u'%s' % self.name


    class Meta:
        ordering = ['ordering',]
        verbose_name = _('carrier')
        verbose_name_plural = _('carriers')


    def get_zone(self, country):
        try:
            return self.zones.filter(countries=country).get()
        except Zone.DoesNotExist:
            if self.default_zone:
                return self.default_zone


class Zone(models.Model):
    carrier = models.ForeignKey(Carrier, verbose_name=_('carrier'), related_name='zones')
    name = models.CharField(_('name'), max_length=50)
    countries = models.ManyToManyField(Country, verbose_name=_('countries'), blank=True)
    handling = models.DecimalField(_('handling'), max_digits=10, decimal_places=2,
        null=True, blank=True)


    def __unicode__(self):
        return u'%s' % self.name


    class Meta:
        unique_together = ('carrier', 'name')
        ordering = ['carrier', 'name',]
        verbose_name = _('zone')
        verbose_name_plural = _('zones')


    def _find_translation(self, language_code=None):
        if not language_code:
            language_code = get_language()
    
        c = self.translations.filter(lang_code__exact=language_code)
        ct = c.count()
    
        if not c or ct == 0:
            pos = language_code.find('-')
            if pos > -1:
                short_code = language_code[:pos]
                log.debug("%s: Trying to find root language content for: [%s]", self, short_code)
                c = self.translations.filter(lang_code__exact=short_code)
                ct = c.count()
                if ct > 0:
                    log.debug("%s: Found root language content for: [%s]", self, short_code)
    
        if not c or ct == 0:
            #log.debug("Trying to find default language content for: %s", self)
            c = self.translations.filter(lang_code__istartswith=settings.LANGUAGE_CODE)
            ct = c.count()
    
        if not c or ct == 0:
            #log.debug("Trying to find *any* language content for: %s", self)
            c = self.translations.all()
            ct = c.count()
    
        if ct > 0:
            return c[0]
        else:
            return None


    def delivery(self):
        """
        Get the delivery, looking up by language code, falling back gracefully
        """
        trans = self._find_translation()
        if trans:
            return trans.delivery
        else:
            return u''
    delivery = property(delivery)


    def description(self):
        """
        Get the description, looking up by language code, falling back gracefully
        """
        trans = self._find_translation()
        if trans:
            return trans.description
        else:
            return u''
    description = property(description)


    def method(self):
        """
        Get the description, looking up by language code, falling back gracefully
        """
        trans = self._find_translation()
        if trans:
            return trans.method
        else:
            return u''
    method = property(method)


    def cost(self, weight):
        """
        Get a price for this weight
        """
        tiers_tmp = self.tiers.filter(min_weight__gte=weight).order_by('min_weight')
        tiers = tiers_tmp.filter(expires__gte=date.today())[:1]

        if tiers.count() is 0:
            tiers = tiers_tmp.filter(expires__isnull=True)[:1]

        if tiers.count() is not 0:
            return tiers[0].cost
        else:
            log.debug("No tiered price found for %s: weight=%s", self, weight)
            raise TieredWeightException


class ZoneTranslation(models.Model):
    zone = models.ForeignKey(Zone, verbose_name=_('zone'), related_name='translations')
    lang_code = models.CharField(_('language'), max_length=10, choices=settings.LANGUAGES)
    description = models.CharField(_('description'), max_length=200)
    method = models.CharField(_('method'), help_text=_('i.e. Air, Land, Sea'), max_length=200)
    delivery = models.CharField(_('delivery'), max_length=200)


    def __unicode__(self):
        return u'%s' % self.lang_code


    class Meta:
        ordering = ['lang_code',]
        verbose_name = _('zone translation')
        verbose_name_plural = _('zone translations')


class WeightTier(models.Model):
    zone = models.ForeignKey(Zone, verbose_name=_('zone'), related_name='tiers')
    min_weight = models.DecimalField(_('min weight'), max_digits=10, decimal_places=2)
    handling = models.DecimalField(_('handling ajustment'), max_digits=10, decimal_places=2,
        null=True, blank=True)
    price = models.DecimalField(_('shipping price'), max_digits=10, decimal_places=2)
    expires = models.DateField(_('expires'), null=True, blank=True)
    

    def __unicode__(self):
        return u'Weight: %s (Total cost: %s)' % (self.min_weight, self.cost)


    class Meta:
        unique_together = ('zone', 'min_weight', 'expires')
        ordering = ['min_weight',]
        verbose_name = _('weight tier')
        verbose_name_plural = _('weight tiers')


    def cost(self):
        handling = Decimal('0.0')

        if self.zone.handling:
            handling = handling + self.zone.handling

        if self.handling:
            handling = handling + self.handling

        return Decimal(handling + self.price)
    cost = property(cost)


import config



from livesettings import *
from django.utils.translation import ugettext_lazy as _

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_AUTOSUCCESS', 
    _('Payment Auto Success Module Settings'), 
    ordering = 100)

config_register_list(    
    BooleanValue(PAYMENT_GROUP, 
        'LIVE', 
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode"),
        default=False),
        
    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'payment.modules.autosuccess'),
        
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'AUTOSUCCESS'),

    StringValue(PAYMENT_GROUP,
        'LABEL',
        description=_('English name for this group on the checkout screens'),
        default = 'Pay Now',
        help_text = _('This will be passed to the translation utility')),

    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = '^auto/'),
        
    BooleanValue(PAYMENT_GROUP,
        'EXTRA_LOGGING',
        description=_("Verbose logs"),
        help_text=_("Add extensive logs during post."),
        default=False)
)



"""
This module provides a simple mecanism for getting a per-thread 
contextual session object.

For basic usage, simply import the ``ScopedSession`` object and
call ``set_session`` on a bound instance. 

The ``set_session`` argument requires a ``engine`` object as its first argument. 
Once this step is done the scoped session object should be bound to your engine 
and ready to use. 
"""
 
from sqlalchemy.orm import scoped_session, sessionmaker

__all__ = ['ScopedSession']

class ScopedSession(object):
    """  Contextual session wrapper """

    initialized = False
 
    def __init__(self, engine=None, **kwargs):
        
        self._session = None
        
        if engine is not None:
            self._engine = engine
            self.set_session(engine, **kwargs)
            self.initialized = True

    def get_session(self):
        """ Returns a Session instance object. """
        return getattr(self, '_session')

    def set_session(self, engine, autoflush=True, autocommit=False):
        """ Set up a default Session instance object. """
        local_session = scoped_session(sessionmaker(autoflush=autoflush, \
            autocommit=autocommit))
        # configure the Session object
        local_session.configure(bind=engine)
        self._session = local_session
        #self.initialized = True
        del local_session
        

    session = property(get_session, set_session)




#
#   SERMEPA / ServiRed payments module for Satchmo
#
#   Author: Michal Salaban <michal (at) salaban.info>
#   with a great help of Fluendo S.A., Barcelona
#
#   Based on "Guia de comercios TPV Virtual SIS" ver. 5.18, 15/11/2008, SERMEPA
#   For more information about integration look at http://www.sermepa.es/
#
#   TODO: SERMEPA interface provides possibility of recurring payments, which
#   could be probably used for SubscriptionProducts. This module doesn't support it.
#
from django.core import urlresolvers
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseNotFound, HttpResponseBadRequest
from django.shortcuts import render_to_response
from django.views.decorators.cache import never_cache
from django.template import RequestContext
from livesettings import config_get_group, config_value 
from payment.utils import get_processor_by_key
from payment.views import payship
from satchmo_store.shop.models import Order, Cart
from satchmo_utils.dynamic import lookup_url, lookup_template
from django.utils.translation import ugettext_lazy as _

from datetime import datetime
from decimal import Decimal
import logging
try:
    from hashlib import sha1
except ImportError:
    # python < 2.5
    from sha import sha as sha1

log = logging.getLogger()

def pay_ship_info(request):
    return payship.base_pay_ship_info(
            request,
            config_get_group('PAYMENT_SERMEPA'), payship.simple_pay_ship_process_form,
            'shop/checkout/sermepa/pay_ship.html'
            )
pay_ship_info = never_cache(pay_ship_info)


def _resolve_local_url(payment_module, cfgval, ssl=False):
    try:
        return lookup_url(payment_module, cfgval.value, include_server=True, ssl=ssl)
    except urlresolvers.NoReverseMatch:
        return cfgval.value


def confirm_info(request):
    payment_module = config_get_group('PAYMENT_SERMEPA')

    try:
        order = Order.objects.from_request(request)
    except Order.DoesNotExist:
        url = lookup_url(payment_module, 'satchmo_checkout-step1')
        return HttpResponseRedirect(url)

    tempCart = Cart.objects.from_request(request)
    if tempCart.numItems == 0:
        template = lookup_template(payment_module, 'shop/checkout/empty_cart.html')
        return render_to_response(template,
                                  context_instance=RequestContext(request))

    # Check if the order is still valid
    if not order.validate(request):
        context = RequestContext(request, {'message': _('Your order is no longer valid.')})
        return render_to_response('shop/404.html', context_instance=context)

    # Check if we are in test or real mode
    live = payment_module.LIVE.value
    if live:
        post_url = payment_module.POST_URL.value
        signature_code = payment_module.MERCHANT_SIGNATURE_CODE.value
        terminal = payment_module.MERCHANT_TERMINAL.value
    else:
        post_url = payment_module.POST_TEST_URL.value
        signature_code = payment_module.MERCHANT_TEST_SIGNATURE_CODE.value
        terminal = payment_module.MERCHANT_TEST_TERMINAL.value

    # SERMEPA system does not accept multiple payment attempts with the same ID, even
    # if the previous one has never been finished. The worse is that it does not display
    # any message which could be understood by an end user.
    #
    # If user goes to SERMEPA page and clicks 'back' button (e.g. to correct contact data),
    # the next payment attempt will be rejected.
    #
    # To provide higher probability of ID uniqueness, we add mm:ss timestamp part
    # to the order id, separated by 'T' character in the following way:
    #
    #   ID: oooooooTmmss
    #   c:  123456789012
    #
    # The Satchmo's Order number is therefore limited to 10 million - 1.
    now = datetime.now()
    xchg_order_id = "%07dT%02d%02d" % (order.id, now.minute, now.second)

    amount = "%d" % (order.balance * 100,)    # in cents
    signature_data = ''.join(
            map(str, (
                    amount,
                    xchg_order_id,
                    payment_module.MERCHANT_FUC.value,
                    payment_module.MERCHANT_CURRENCY.value,
                    signature_code,
                    )
               )
            )

    signature = sha1(signature_data).hexdigest()

    template = lookup_template(payment_module, 'shop/checkout/sermepa/confirm.html')

    url_callback = _resolve_local_url(payment_module, payment_module.MERCHANT_URL_CALLBACK, ssl=payment_module.SSL.value)
    url_ok = _resolve_local_url(payment_module, payment_module.MERCHANT_URL_OK)
    url_ko = _resolve_local_url(payment_module, payment_module.MERCHANT_URL_KO)

    ctx = {
        'live': live,
        'post_url': post_url,
        'MERCHANT_CURRENCY': payment_module.MERCHANT_CURRENCY.value,
        'MERCHANT_FUC': payment_module.MERCHANT_FUC.value,
        'terminal': terminal,
        'MERCHANT_TITULAR': payment_module.MERCHANT_TITULAR.value,
        'url_callback': url_callback,
        'url_ok': url_ok,
        'url_ko': url_ko,
        'order': order,
        'xchg_order_id' : xchg_order_id,
        'amount': amount,
        'signature': signature,
        'default_view_tax': config_value('TAX', 'DEFAULT_VIEW_TAX'),
    }
    return render_to_response(template, ctx, context_instance=RequestContext(request))
confirm_info = never_cache(confirm_info)

def notify_callback(request):
    payment_module = config_get_group('PAYMENT_SERMEPA')
    if payment_module.LIVE.value:
        log.debug("Live IPN on %s", payment_module.KEY.value)
        signature_code = payment_module.MERCHANT_SIGNATURE_CODE.value
        terminal = payment_module.MERCHANT_TERMINAL.value
    else:
        log.debug("Test IPN on %s", payment_module.KEY.value)
        signature_code = payment_module.MERCHANT_TEST_SIGNATURE_CODE.value
        terminal = payment_module.MERCHANT_TEST_TERMINAL.value
    data = request.POST
    log.debug("Transaction data: " + repr(data))
    try:
        sig_data = "%s%s%s%s%s%s" % (
                data['Ds_Amount'],
                data['Ds_Order'],
                data['Ds_MerchantCode'],
                data['Ds_Currency'],
                data['Ds_Response'],
                signature_code
                )
        sig_calc = sha1(sig_data).hexdigest()
        if sig_calc != data['Ds_Signature'].lower():
            log.error("Invalid signature. Received '%s', calculated '%s'." % (data['Ds_Signature'], sig_calc))
            return HttpResponseBadRequest("Checksum error")
        if data['Ds_MerchantCode'] != payment_module.MERCHANT_FUC.value:
            log.error("Invalid FUC code: %s" % data['Ds_MerchantCode'])
            return HttpResponseNotFound("Unknown FUC code")
        if int(data['Ds_Terminal']) != int(terminal):
            log.error("Invalid terminal number: %s" % data['Ds_Terminal'])
            return HttpResponseNotFound("Unknown terminal number")
        # TODO: fields Ds_Currency, Ds_SecurePayment may be worth checking

        xchg_order_id = data['Ds_Order']
        try:
            order_id = xchg_order_id[:xchg_order_id.index('T')]
        except ValueError:
            log.error("Incompatible order ID: '%s'" % xchg_order_id)
            return HttpResponseNotFound("Order not found")
        try:
            order = Order.objects.get(id=order_id)
        except Order.DoesNotExist:
            log.error("Received data for nonexistent Order #%s" % order_id)
            return HttpResponseNotFound("Order not found")
        amount = Decimal(data['Ds_Amount']) / Decimal('100')    # is in cents, divide it
        if int(data['Ds_Response']) > 100:
            log.info("Response code is %s. Payment not accepted." % data['Ds_Response'])
            return HttpResponse()
    except KeyError:
        log.error("Received incomplete SERMEPA transaction data")
        return HttpResponseBadRequest("Incomplete data")
    # success
    order.add_status(status='New', notes=u"Paid through SERMEPA.")
    processor = get_processor_by_key('PAYMENT_SERMEPA')
    payment = processor.record_payment(
        order=order, 
        amount=amount, 
        transaction_id=data['Ds_AuthorisationCode'])
    # empty customer's carts
    for cart in Cart.objects.filter(customer=order.contact):
        cart.empty()
    return HttpResponse()



#!/usr/bin/env python
from notmm.utils.configparse import setup_all, is_equal
import logging_conf

global_conf = {}

setup_all('sandbox', global_conf)
#setup_all('logging', global_conf)# logging options

#logconf = global_conf['logging']
#if is_equal(logconf['enabled'], 'true'):
#    logger = getattr(logging_conf, 'logger_func')
#    logger('Logging subsystem started!')
#print global_conf['logging']



from django.contrib.comments.models import Comment
from django.db import models
from django.utils.translation import ugettext, ugettext_lazy as _
from signals_ahoy.signals import collect_urls
import product
import satchmo_store

class ProductRating(models.Model):
    """A rating attached to a comment"""
    comment = models.OneToOneField(Comment, verbose_name="Rating", primary_key=True)
    rating = models.IntegerField(_("Rating"))

import config
from urls import add_product_urls, add_comment_urls
collect_urls.connect(add_product_urls, sender=product)
collect_urls.connect(add_comment_urls, sender=satchmo_store)


from django import forms
from django.conf import settings
from django.contrib.auth.decorators import permission_required
from django.contrib.auth.decorators import user_passes_test
from django.core import urlresolvers
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from keyedcache.models import *
import logging

log = logging.getLogger('keyedcache.views')

YN = (
    ('Y', _('Yes')),
    ('N', _('No')),
    )

class CacheDeleteForm(forms.Form):
    tag = forms.CharField(label=_('Key to delete'), required=False)
    children = forms.ChoiceField(label=_('Include Children?'), choices=YN, initial="Y")
    kill_all = forms.ChoiceField(label=_('Delete all keys?'), choices=YN, initial="Y")
    
    def delete_cache(self):
        
        data = self.cleaned_data
        if data['kill_all'] == "Y":
            keyedcache.cache_delete()
            result = "Deleted all keys"
        elif data['tag']:
            keyedcache.cache_delete(data['tag'], children=data['children'])
            if data['children'] == "Y":
                result = "Deleted %s and children" % data['tag']
            else:
                result = "Deleted %s" % data['tag']
        else:
            result = "Nothing selected to delete"
        
        log.debug(result)
        return result

def stats_page(request):
    calls = keyedcache.CACHE_CALLS
    hits = keyedcache.CACHE_HITS
    
    if (calls and hits):
        rate =  float(keyedcache.CACHE_HITS)/keyedcache.CACHE_CALLS*100
    else:
        rate = 0
        
    try:
        running = keyedcache.cache_require()
        
    except keyedcache.CacheNotRespondingError:
        running = False
        
    ctx = RequestContext(request, {
        'cache_count' : len(keyedcache.CACHED_KEYS),
        'cache_running' : running,
        'cache_time' : settings.CACHE_TIMEOUT,
        'cache_backend' : settings.CACHE_BACKEND,
        'cache_calls' : keyedcache.CACHE_CALLS,
        'cache_hits' : keyedcache.CACHE_HITS,
        'hit_rate' : "%02.1f" % rate
    })

    return render_to_response('keyedcache/stats.html', context_instance=ctx)

stats_page = user_passes_test(lambda u: u.is_authenticated() and u.is_staff, login_url='/accounts/login/')(stats_page)
    
def view_page(request):
    keys = keyedcache.CACHED_KEYS.keys()
    
    keys.sort()
    
    ctx = RequestContext(request, { 
        'cached_keys' : keys,
    })

    return render_to_response('keyedcache/view.html', context_instance=ctx)

view_page = user_passes_test(lambda u: u.is_authenticated() and u.is_staff, login_url='/accounts/login/')(view_page)

def delete_page(request):
    log.debug("delete_page")
    if request.method == "POST":
        form = CacheDeleteForm(request.POST)
        if form.is_valid():
            log.debug('delete form valid')
            results = form.delete_cache()
            return HttpResponseRedirect('../')
        else:
            log.debug("Errors in form: %s", form.errors)
    else:
        log.debug("new form")
        form = CacheDeleteForm()
            
    ctx = RequestContext(request, { 
        'form' : form,
    })

    return render_to_response('keyedcache/delete.html', context_instance=ctx)

delete_page = user_passes_test(lambda u: u.is_authenticated() and u.is_staff, login_url='/accounts/login/')(delete_page)



# -*- coding: utf-8 -*-

import os
import urlparse
from decimal import Decimal
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from livesettings import config_register, BooleanValue, StringValue, \
    ConfigurationGroup, PositiveIntegerValue, \
    DecimalValue


SHOP_GROUP = ConfigurationGroup('SHOP', _('Satchmo Shop Settings'), ordering=0)

default_icon_url = urlparse.urlunsplit(
    ('file',
     '',
     os.path.join(settings.MEDIA_ROOT, 'images/sample-logo.bmp'),
     '',
     '')
    )

#### SHOP Group ####

LOGO_URI = config_register(
    StringValue(SHOP_GROUP,
    'LOGO_URI',
    description = _("URI to the logo for the store"),
    help_text = _(("For example http://www.example.com/images/logo.jpg or "
                   "file:///var/www/html/images/logo.jpg")),
    default = default_icon_url))

ENFORCE_STATE = config_register(
    BooleanValue(SHOP_GROUP,
    'ENFORCE_STATE',
    description = _('State required?'),
    help_text = _("Require a state during registration/checkout for countries that have states?"),
    default = True))

SHOW_SITE = config_register(
    BooleanValue(SHOP_GROUP,
    'SHOW_SITE',
    description = _('Show Site Field?'),
    help_text = _("Should the Site field be displayed in the admin lists? A server restart is required for this to take effect."),
    default = True))

config_register(DecimalValue(
    SHOP_GROUP,
        'CART_ROUNDING',
        description = _('Cart Quantity Rounding Factor'),
        help_text = _("What to round cart adds/deletes by, a '1' here means to round up to a whole number.  Must be -1 to 1."),
        default = Decimal('1')
    ))

config_register(PositiveIntegerValue(
    SHOP_GROUP,
        'CART_PRECISION',
        description = _('Cart Quantity Decimal Places'),
        help_text = _("How many places to assume for cart quantities, use 0 unless you are selling products in fractional quantities."),
        default = 0
    ))

PERSISTENT_CART = config_register(
    BooleanValue(SHOP_GROUP,
    'PERSISTENT_CART',
    description = _('Persistent Cart?'),
    help_text = _("When a user logs in, attempt to retrieve previous carts and merge with existing?"),
    default = False
    ))

HTML_EMAIL = config_register(
    BooleanValue(SHOP_GROUP,
    'HTML_EMAIL',
    description = _('Send HTML Email?'),
    help_text = _("Should HTML emails be used when sending notifications?"),
    default = False
    ))

CART_QTY = config_register(
    BooleanValue(SHOP_GROUP,
    'CART_QTY',
    description=_('Use Cart Price Breaks?'),
    help_text=_('Use the total number or cart or order items to determine quantities for discount pricing.'),
    default=False
    ))


#### Google Group ####

GOOGLE_GROUP = ConfigurationGroup('GOOGLE', _('Google Settings'))

GOOGLE_ANALYTICS = config_register(
    BooleanValue(GOOGLE_GROUP,
        'ANALYTICS',
        description= _("Enable Analytics"),
        default=False,
        ordering=0))

GOOGLE_USE_URCHIN = config_register(
    BooleanValue(GOOGLE_GROUP,
        'USE_URCHIN',
        description= _("Use Urchin?"),
        help_text=_("Use the old-style, urchin javascript?.  This is not needed unless your analytics account hasn't been updated yet."),
        default = False,
        ordering=5,
        requires = GOOGLE_ANALYTICS))

GOOGLE_ANALYTICS_CODE = config_register(
    StringValue(GOOGLE_GROUP,
        'ANALYTICS_CODE',
        description= _("Analytics Code"),
        default = "",
        ordering=5,
        requires = GOOGLE_ANALYTICS))

GOOGLE_ADWORDS = config_register(
    BooleanValue(GOOGLE_GROUP,
        'ADWORDS',
        description= _("Enable Conversion Tracking"),
        default=False,
        ordering=10))

GOOGLE_ADWORDS_ID = config_register(
    StringValue(GOOGLE_GROUP,
        'ADWORDS_ID',
        description= _("Adwords ID (ex: UA-abcd-1)"),
        default = "",
        ordering=15,
        requires = GOOGLE_ADWORDS))



# Suck in default django settings
from django.conf.global_settings import *
from pkg_resources import resource_filename

DEBUG = True
USE_I18N = True

DEFAULT_CHARSET = 'UTF-8'
DEFAULT_CONTENT_TYPE = 'text/html; charset=%s' % DEFAULT_CHARSET

# Set this for debugging the template loader 
TEMPLATE_LOADERS_DEBUG = True

# List of processors used by RequestContext to populate the context.
# Each one should be a callable that takes the request object as its
# only parameter and returns a dictionary to add to the context.
TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.auth',
    'django.core.context_processors.request',
    'django.core.context_processors.media',
    #'django.core.context_processors.debug'
)

# For backward-compatibility with Django
TEMPLATE_DIRS = (
    resource_filename('sandbox', 'templates'),
)

DATABASE_ENGINE = ''
DATABASE_NAME = 'sqlite:///fixtures/test.db'
#DATABASE_OPTIONS = {}
#DATABASE_ECHO = False

ROOT_URLCONF = 'urls'




from decimal import Decimal, InvalidOperation
from django.contrib.sites.models import Site
from django.db.models import Q
from product.models import Product, Category, Discount
import logging

log = logging.getLogger('search listener')

def default_product_search_listener(sender, request=None, category=None, keywords=[], results={}, **kwargs):
    """Performs the base satchmo search.  This is easily overridden by unregistering the listener and creating your own.
    However, it usually won't have to be overridden, since it just adds data to the results dict.  If you are simply
    adding more results, then leave this listener registered and add more objects in your search listener.
    """
    log.debug('default product search listener')
    site = Site.objects.get_current()
    products = Product.objects.all()
    productkwargs = {
        #'productvariation__parent__isnull' : True,
        'active' : True,
        'site' : site
    }

    if category:
        categories = Category.objects.active().filter(slug=category)
        if categories:
            categories = categories[0].get_active_children(include_self=True)
        
        productkwargs['category__in'] = categories
    else:
        categories = Category.objects.active()

    for keyword in keywords:
        if not category:
            categories = categories.filter(
                Q(name__icontains=keyword) |
                Q(meta__icontains=keyword) |
                Q(description__icontains=keyword),
                site=site,)
                
        products = products.filter(
            Q(name__icontains=keyword)
            | Q(short_description__icontains=keyword)
            | Q(description__icontains=keyword)
            | Q(meta__icontains=keyword)
            | Q(sku__iexact=keyword),
            **productkwargs)

    results.update({
        'categories': categories, 
        'products': products
        })

def priceband_search_listener(sender, request=None, category=None, keywords=[], results={}, **kwargs):
    """Filter search results by price bands.
    
    If a "priceband" parameter is available, it will be parsed as follows:
        lowval-highval
        if there is no "-", then it will be parsed as lowval or higher
    """
    log.debug('priceband search listener')
    if request.method=="GET":
        data = request.GET
    else:
        data = request.POST
    
    priceband = data.get('priceband', None)
    
    if not priceband:
        return
        
    bands = priceband.split('-')
    try:
        if len(bands) > 1:
            low = int(bands[0])
            high = int(bands[1])
        else:
            low = bands[0]
            high = Decimal('1000000000.00')
    except (TypeError, InvalidOperation):
        log.warn("Couldn't parse priceband=%s", priceband)
        return
    
    priced = []
    products = results['products']
    products = products.filter(productvariation__parent__isnull=True)
    log.debug('priceband pre-filter length=%i', len(products))
    for p in products:
        price = p.unit_price
        if price>low and price<=high:
            priced.append(p)
    log.debug('priceband post-filter length=%i', len(priced))
    
    categories = results['categories']
    categories = categories.filter(product__in = priced)
    results['products'] = priced
    results['categories'] = categories

def discount_used_listener(sender, order=None, **kwargs):
    """If an order has a discount, increment numUses on it.
    
    satchmo_store.shop.signals.order_success listener set up in shop.listeners.
    """
    if order.discount_code:
        try:
            discount = Discount.objects.by_code(order.discount_code)
            if discount.numUses:
                discount.numUses += 1
            else:
                discount.numUses = 1
            discount.save()
        except Discount.DoesNotExist:
            pass



from notmm.utils.wsgilib.exc import HTTPRedirectResponse
from notmm.utils.template import direct_to_template

import logging
log = logging.getLogger('notmm.controllers.wsgi')

__all__ = ('handle302', 'handle303', 'handle404', 'handle500')

def handle302(request, **kwargs):
    location = kwargs['location']
    return HTTPRedirectResponse(location=location, status=302)

def handle303(request, **kwargs):
    location = kwargs['location']
    return HTTPRedirectResponse(location=location, status=303)

def handle500(request, template_name='500.mako'):
    #exc = request.environ['wsgi.exception']
    #log.debug('caught a server error: %s !'%request.path_url)
    return direct_to_template(request, template_name, status=500)
 
def handle404(request, template_name='404.mako'):
    #self.logger.info('Location=%r' % request.path_url)
    #self.logger.info('Client IP=%r' % request.remote_addr)
    return direct_to_template(request, template_name, status=404)
def handle401(request, template_name='401.mako'):
    return direct_to_template(request, template_name, status=401)



from django.contrib.sites.models import Site
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from models import Brand, BrandCategory, BrandProduct
from product import signals
from product.models import Product
from product.utils import find_best_auto_discount

import logging

log = logging.getLogger("satchmo_brand.views")

def brand_list(request):
    brands = Brand.objects.active()
    ctx = {
        'brands' : brands,
    }
    signals.index_prerender.send(Brand, request=request, context=ctx, object_list=brands)
    requestctx = RequestContext(request, ctx)
    return render_to_response('brand/index.html',
                              context_instance=requestctx)

def brand_page(request, brandname):
    try:
        brand = Brand.objects.by_slug(brandname)
        
    except Brand.DoesNotExist:
        raise Http404(_('Brand "%s" does not exist') % brandname)

        
    products = list(brand.active_products())
    sale = find_best_auto_discount(products)

    ctx = {
        'brand' : brand,
        'products': products,
        'sale' : sale,
    }

    ctx = RequestContext(request, ctx)
    signals.index_prerender.send(BrandProduct, request=request, context=ctx, brand=brand, object_list=products)

    return render_to_response('brand/view_brand.html',
                              context_instance=ctx)


def brand_category_page(request, brandname, catname):
    try:
        cat = BrandCategory.objects.by_slug(brandname, catname)
        
    except Brand.DoesNotExist:
        raise Http404(_('Brand "%s" does not exist') % brandname)
        
    except BrandCategory.DoesNotExist:
        raise Http404(_('No category "%{category}s" in brand "%{brand}s"').format(category=catname, brand=brandname))
        
    products = list(cat.active_products())
    sale = find_best_auto_discount(products)
    
    ctx = RequestContext(request, {
        'brand' : cat,
        'sale' : sale,
    })
    return render_to_response('brand/view_brand.html', context_instance=ctx)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2013 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved.
#
# <LICENSE=APACHEV2>
#
"""Misc utilities for parsing configuration files in RFC-822 format."""

import os
import sys
import posixpath
from configobj import ConfigObj

__all__ = ('string_getter', 'int_getter', 'long_getter', 
           'is_string', 'is_utf8', 'make_path',
           'has_conf', 'is_equal', 'is_uppercase', 
           'loadconf', 'getconf', 'make_path', 'setup_all',
           'as_bool', 'paste_eval_import', 'eval_import')

# conf_getters like functions
string_getter = lambda conf,section,key: str(conf[section].get(key, None))

# builtin numeric-like getters: wtf?
# i don't remember using thoses for a while.. :)
int_getter = lambda *s: int(string_getter(*s))
long_getter = lambda *s: long(int_getter(*s))

### more string-like hooks; 
#this one returns a boolean
is_string = lambda s: bool(isinstance(s, str))
# verifies if the string is encoded in utf8 format
is_utf8 = lambda s: bool(isinstance(s, unicode))
### "path" functions
make_path = lambda relname: posixpath.realpath(posixpath.abspath(relname))
has_conf = lambda x, y: os.access (posixpath.join (x, y), os.R_OK)
### logic related test functions
is_equal = lambda a, b: bool(a==b)
is_uppercase = lambda s: bool(s.isupper())

# Emulate ``paste.util.converters.asbool`` but using 
# ``configobj.as_bool`` semantics.
as_bool = lambda cfg, v: cfg.as_bool(v)

def loadconf(filename, section=None, config_class=ConfigObj):
    """ 
    Returns a fully configured ConfigObj instance.

    """
    app_conf = config_class(make_path(filename))
    if section in app_conf:
        return app_conf[section]
    else:
        return app_conf   

def getconf(relname):
    """Loads a configuration file and return its content."""
    # Default options are now located in development.ini.
    return loadconf(relname)

def setup_all(key, global_conf, c='development.ini'):
    """
    Loads development (config) options required for running an
    application and put them in the application global 
    configuration dict, as referenced by ``key``, or else 
    into a similar persistent storage object.

    Returns ``False`` on success, or ``True`` if no development.ini
    file has been found.

    This function is optional, and should not be necessary 
    for normal operations. 
    """
    
    if has_conf(os.getcwd(), c):
        # Add a reference to ``key`` in the dict instance ``global_conf`` 
        global_conf.update({key: loadconf(c, section=key).dict()})
        return False
    return True


def paste_eval_import(name):
    """Helper function to avoid installing Paste as requirement"""
    from pastelib import eval_import
    return eval_import(name)

eval_import = lambda s: paste_eval_import(s)



from decimal import Decimal
from django.core.exceptions import ImproperlyConfigured
from django.db.models import Q
from l10n.models import AdminArea, Country
from livesettings import config_value
from models import TaxRate
from product.models import TaxClass
from satchmo_store.contact.models import Contact
from satchmo_utils import is_string_like
import logging

log = logging.getLogger('tax.area')

class Processor(object):
    
    method = "area"
    
    def __init__(self, order=None, user=None):
        """
        Any preprocessing steps should go here
        For instance, copying the shipping and billing areas
        """
        self.order = order
        self.user = user
        
    def _get_location(self):
        area=country=None
        calc_by_ship_address = bool(config_value('TAX','TAX_AREA_ADDRESS') == 'ship')
        if self.order:
            if calc_by_ship_address:
                country = self.order.ship_country
                area = self.order.ship_state
            else:
                country = self.order.bill_country
                area = self.order.bill_state
        
            if country:
                try:
                    country = Country.objects.get(iso2_code__exact=country)
                except Country.DoesNotExist:
                    log.info("Couldn't find Country from string: %s", country)
                    country = None
        
        elif self.user and self.user.is_authenticated():
            try:
                contact = Contact.objects.get(user=self.user)
                try:
                    if calc_by_ship_address:
                        area = contact.shipping_address.state
                    else:
                        area = contact.billing_address.state
                except AttributeError:
                    pass
                try:
                    if calc_by_ship_address:
                        country = contact.shipping_address.country
                    else:
                        country = contact.billing_address.country
                except AttributeError:
                    pass

            except Contact.DoesNotExist:
                pass

        if not country:
            from satchmo_store.shop.models import Config
            country = Config.objects.get_current().sales_country

        if area:
            try:
                area = AdminArea.objects.get(name__iexact=area,
                                             country=country)
            except AdminArea.DoesNotExist:
                try:
                    area = AdminArea.objects.get(abbrev__iexact=area,
                                                 country=country)
                except AdminArea.DoesNotExist:
                    log.info("Couldn't find AdminArea from string: %s", area)
                    area = None


        return area, country
        
    def get_percent(self, taxclass="Default", area=None, country=None):
        return 100*self.get_rate(taxclass=taxclass, area=area, country=country)
        
    def get_rate(self, taxclass=None, area=None, country=None, get_object=False, **kwargs):
        if not taxclass:
            taxclass = "Default"
        rate = None
        if not (area or country):
            area, country = self._get_location()
            
        if is_string_like(taxclass):
            try:
                taxclass = TaxClass.objects.get(title__iexact=taxclass)
            
            except TaxClass.DoesNotExist:
                raise ImproperlyConfigured("Can't find a '%s' Tax Class", taxclass)            
            
        if area:
            try:
                rate = TaxRate.objects.get(taxClass=taxclass, taxZone=area)
                
            except TaxRate.DoesNotExist:
                rate = None
                
        if not rate:
            try:
                rate = TaxRate.objects.get(taxClass=taxclass, taxCountry=country)
                
            except TaxRate.DoesNotExist:
                rate = None
        
        log.debug("Got rate [%s] = %s", taxclass, rate)
        if get_object:
            return rate
        else:
            if rate:
                return rate.percentage
            else:
                return Decimal("0.00")

    def by_price(self, taxclass, price):
        rate = self.get_rate(taxclass)

        if not rate:
            t = Decimal("0.00")
        else:
            t = rate * price

        return t

    def by_product(self, product, quantity=Decimal('1')):
        """Get the tax for a given product"""
        price = product.get_qty_price(quantity)
        tc = product.taxClass
        return self.by_price(tc, price)
        
    def by_orderitem(self, orderitem):
        if orderitem.product.taxable:
            price = orderitem.sub_total
            taxclass = orderitem.product.taxClass
            return self.by_price(taxclass, price)
        else:
            return Decimal("0.00")

    def shipping(self, subtotal=None):
        if subtotal is None and self.order:
            subtotal = self.order.shipping_sub_total

        if subtotal:
            rate = None
            if config_value('TAX','TAX_SHIPPING'):
                try:
                    tc = TaxClass.objects.get(title=config_value('TAX', 'TAX_CLASS'))
                    rate = self.get_rate(taxclass=tc)
                except:
                    log.error("'Shipping' TaxClass doesn't exist.")

            if rate:
                t = rate * subtotal
            else:
                t = Decimal("0.00")
            
        else:
            t = Decimal("0.00")
        
        return t

    def process(self, order=None):
        """
        Calculate the tax and return it.
        
        Probably need to make a breakout.
        """
        if order:
            self.order = order
        else:
            order = self.order
        
        sub_total = Decimal('0.00')
        taxes = {}
        
        rates = {}
        for item in order.orderitem_set.filter(product__taxable=True):
            tc = item.product.taxClass
            if tc:
                tc_key = tc.title
            else:
                tc_key = "Default"
                
            if rates.has_key(tc_key):
                rate = rates[tc_key]
            else:
                rate = self.get_rate(tc, get_object=True)
                rates[tc_key] = rate
                taxes[tc_key] = Decimal("0.00")
                
            price = item.sub_total
            if rate:
                t = price*rate.percentage
            else:
                t = Decimal("0.00")
            sub_total += t
            taxes[tc_key] += t
        
        ship = self.shipping()
        sub_total += ship
        taxes['Shipping'] = ship
        
        for k in taxes:
            taxes[k] = taxes[k]
        
        return sub_total, taxes
